##
## undoModelUpdates.py
## - Remove the Service Policy Violation's Attributes.
## - Remove the JMS Work Classes.
##

# import os
import sys

import java.lang.Exception as JavaException
import java.io.File as JavaFile

##
# Convert Str to List
##
def convertToList(inlist):
    outlist = []
    if (len(inlist) > 0 and inlist[0] == '[' and inlist[len(inlist) - 1] == ']'):
        inlist = inlist[1:len(inlist) - 1]
        clist = inlist.split(" ")
    else:
        clist = inlist.split("\n")
    for elem in clist:
        elem = elem.rstrip();
        if (len(elem) > 0):
            outlist.append(elem)
    return outlist

##
# Backup a File
##
def backupFile(src, dst):
    if traceON: print "> backupFile()"

    srcFile = open(src, 'r')
    dstFile = open(dst, 'w')

    # for line in srcFile:
    #    dstFile.write(line)

    aLine = srcFile.readline()
    while aLine:
        dstFile.write(aLine)
        aLine = srcFile.readline()
    dstFile.write(aLine)

    srcFile.close()
    dstFile.close()

    if traceON: print "< backupFile()"

##
# Backup Location
##
def backupLocation():
    return wasHome + "/properties/version/uninstall.xd/backup.operationalpolicy"

##
# Backup XML
##
def backupXML(item):
    docDir = "config/%s" % parseDocumentContext(item)
    if traceON: print docDir
    docPath = "%s/serviceclass.xml" % docDir
    if traceON: print docPath
    srcLocation = profilePath + "/" + docPath
    if traceON: print srcLocation
    backupDirPath = "%s/%s/%s" % (backupLocation(), profileName, docDir)
    if traceON: print backupDirPath
    dstLocation = "%s/%s/%s" % (backupLocation(), profileName, docPath)
    if traceON: print dstLocation

    try:
        # os.makedirs(backupDirPath)
        backupDir = JavaFile(backupDirPath)
        if not backupDir.exists():
            if backupDir.mkdirs():
                backupFile(srcLocation, dstLocation)
                print "Backup created at %s." % dstLocation
            else:
                print "Failed to create Directory, Skip Backup."
        else:
            backupFile(srcLocation, dstLocation)
            print "Backup created at %s." % dstLocation
    except Exception, e:
        print "Exception: %s" % e
        print sys.exc_info()
        print "In Exception, Skip Backup Creation."

##
# Create Non-SPV Service Class
##
def createNonSPVServiceClass(aDataSet):
    print ".. Create Service Class without Violation Attributes"
    print ".. Service Class Data: ", aDataSet

    theCellPath = "/Cell:%s" % aDataSet['Cell']
    theCell = AdminConfig.getid(theCellPath)
    scAttributes = [ ["name", aDataSet['scName']], ["description", aDataSet['scDesc']] ]
    theServiceClass = AdminConfig.create("ServiceClass", theCell, scAttributes)
    print ".... Create Service Class"

    # Service Class Goal
    if aDataSet['scType'] == "DiscretionaryGoal":
        AdminConfig.create("DiscretionaryGoal", theServiceClass, [])
    else: 
        theGoalAttrs = [ ["importance", aDataSet['scImportance']], ["goalValue", aDataSet['scGoalValue']],
            ["goalValueUnits", aDataSet['scGoalUnit']] ]

        if aDataSet['scType'] == "PercentileResponseTimeGoal":
            theGoalAttrs.append(["goalPercent", aDataSet['scGoalPercent']])
            AdminConfig.create("PercentileResponseTimeGoal", theServiceClass, theGoalAttrs)
        elif aDataSet['scType'] == "AverageResponseTimeGoal":
            AdminConfig.create("AverageResponseTimeGoal", theServiceClass, theGoalAttrs)
        elif aDataSet['scType'] == "QueueTimeGoal":
            AdminConfig.create("QueueTimeGoal", theServiceClass, theGoalAttrs)
    print ".... Create Performance Goal"

    # Transaction Classes
    theTCs = aDataSet['TCs']
    for elem in theTCs:
        tcAttributes = [ ["name", elem['name']], ["description", elem['desc']] ]
        AdminConfig.create("TransactionClass", theServiceClass, tcAttributes)
    print ".... Create Transaction Classes"

    print ".. Service Class %s Created." % theServiceClass

##
# Read Non-SPV Data from a Current Service Class
##
def readNonSPVData(anID):
    print ".. Backup Data for %s" % anID

    aData = {}
    aData['ID'] = anID
    if traceON: print "ID: %s" % aData['ID']
    aData['Cell'] = checkCellName(anID)
    if traceON: print "Cell: %s" % aData['Cell']

    aData['scName'] = AdminConfig.showAttribute(anID, "name")
    if traceON: print "scName: %s" % aData['scName']
    aData['scDesc'] = AdminConfig.showAttribute(anID, "description")
    if traceON: print "scDesc: %s" % aData['scDesc']

    theSCG = AdminConfig.showAttribute(anID, "ServiceClassGoal")
    if traceON: print "theSCG: %s" % theSCG
    aData['scType'] = checkServiceClassGoalType(theSCG)
    if traceON: print "scType: %s" % aData['scType']
    aData['scImportance'] = AdminConfig.showAttribute(theSCG, "importance")
    if traceON: print "scImportance: %s" % aData['scImportance']

    aData['scGoalValue'] = AdminConfig.showAttribute(theSCG, "goalValue")
    if traceON: print "scGoalValue: %s" % aData['scGoalValue']
    aData['scGoalUnit'] = AdminConfig.showAttribute(theSCG, "goalValueUnits")
    if traceON: print "scGoalUnit: %s" % aData['scGoalUnit']
    if aData['scType'] == "PercentileResponseTimeGoal":
        aData['scGoalPercent'] = AdminConfig.showAttribute(theSCG, "goalPercent")
        if traceON: print "scGoalPercent: %s" % aData['scGoalPercent']

    theTCs = AdminConfig.showAttribute(anID, "TransactionClasses")
    if traceON: print "theTCs: %s" % theTCs
    theTCList = convertToList(theTCs)
    theTCsData = []
    for elem in theTCList:
        tcData = {}
        tcData['name'] = AdminConfig.showAttribute(elem, "name")
        tcData['desc'] = AdminConfig.showAttribute(elem, "description")
        theTCsData.append(tcData)
    aData['TCs'] = theTCsData
    if traceON: print "TCs: %s" % aData['TCs']

    return aData

##
# Determine the Cell's Name
##
def checkCellName(theID):
    startIndex = theID.find("cells/") + 6
    endIndex = theID.find("/serviceclasses/")

    return theID[startIndex:endIndex]

##
# Determine the Service Class' Goal Type
##
def checkServiceClassGoalType(theSCG):
    aRTG = "AverageResponseTimeGoal"
    pRTG = "PercentileResponseTimeGoal"
    qTG = "QueueTimeGoal"

    theGoalType = "DiscretionaryGoal"
    if theSCG.find(aRTG) != -1:
        theGoalType = aRTG
    elif theSCG.find(pRTG) != -1:
        theGoalType = pRTG
    elif theSCG.find(qTG) != -1:
        theGoalType = qTG

    return theGoalType

##
# Parse Document Context from ID
##
def parseDocumentContext(anID):
    if traceON: print "> parseDocumentContext()"

    startIndex = anID.find("(") + 1
    endIndex = anID.find("|")

    if traceON: print "< parseDocumentContext()"

    return anID[startIndex:endIndex]

##
# Undo Service Classes
##
def undoServiceClasses():
    scList = AdminConfig.list("ServiceClass")
    theSCList = convertToList(scList)

    print "%s ServiceClasses Found." % len(theSCList)
    print ""    

    for item in theSCList:
        performAttrRemoval = 0

        theSCG = AdminConfig.showAttribute(item, "ServiceClassGoal")
        try:
            # Find TimePeriodValue to see if Violation Attributes set
            # XXX: Use 'violationEnabled' when available
            AdminConfig.showAttribute(theSCG, "timePeriodValue")
            performAttrRemoval = 1
        except JavaException, e:
            # Failed to retrieve TimePeriodValue
            print "No Service Policy Violation Attributes Found or Failure in Attribute Retrieval."
            if traceON: print str(e)

        theName = AdminConfig.showAttribute(item, "name")
        if performAttrRemoval:
            print "Backup ServiceClass %s's XML .." % theName
            backupXML(item)

            print "Remove Service Policy Violation Attributes from ServiceClass %s .." % theName
            theData = readNonSPVData(item)
            AdminConfig.remove(item)
            createNonSPVServiceClass(theData)
            print ""
        else:
            print "Skip ServiceClass %s." % theName
            print ""

##
# Undo Work Classes
##
def undoWorkClasses():
    wcList = AdminConfig.list("WorkClass")
    theWCList = convertToList(wcList)

    print "%s WorkClasses Found." % len(theWCList)
    print ""

    for item in theWCList:
        theName = AdminConfig.showAttribute(item, "name")
        theType = AdminConfig.showAttribute(item, "type")
        if theType == "JMSWORKCLASS":
            print "Backup WorkClass %s's XML .." % theName
            backupXML(item)

            print "Remove WorkClass %s .." % theName
            AdminConfig.remove(item)
        else:
            print "Not a JMS WorkClass; Skip WorkClass %s." % theName

##
# Counter Location
##
def counterLocation():
    return wasHome + "/properties/version/install.xd/backup.operationalpolicy"

##
# Decrement Counter
##
def decrementCounter(theCount, counterFilePath):
    counterFile = open(counterFilePath, 'w')
    newCount = theCount - 1;
    newCountLine = "%s\n" % newCount
    counterFile.write(newCountLine)
    counterFile.close()

    print "Counter Decremented."
    return newCount

##
# Check and Decrement Counter
##
def checkAndDecrementCounter():
    if traceON: print "> checkAndDecrementCounter()"

    counterFilePath = "%s/counter" % counterLocation()
    counter = JavaFile(counterFilePath)
    newCount = -1

    if counter.exists():
        counterFile = open(counterFilePath, 'r')
        theCount = int(counterFile.readline())
        if traceON: print "Count is %s" % theCount
        counterFile.close()

        if theCount > 1:
            newCount = decrementCounter(theCount, counterFilePath)
            return [0, newCount]

        if theCount == 1:
            newCount = decrementCounter(theCount, counterFilePath)
            if counter.delete():
                print "Counter Deleted."
            else:
                print "FAILED to Delete Counter."
        else:
            return [0, 0]
    else:
        print "Counter Does NOT Exists."
        return [0, newCount]

    if traceON: print "< checkAndDrecementCounter()"

    return [1, newCount]

##--------------------
# Main
##--------------------

global traceON
global wasHome
global profileName
global profilePath

traceON = 0

if len(sys.argv) < 3:
    print "Three Parameters, <WAS_HOME>, <PROFILE_NAME>, and <PROFILE_PATH>, are REQUIRED."
    print "Please retry with the three parameters specified."
else:
    wasHome = sys.argv[0]
    profileName = sys.argv[1]
    profilePath = sys.argv[2]

    if len(sys.argv) == 4:
        traceON = sys.argv[3]

    try:
    #    theCount = checkAndDecrementCounter()
    #    if theCount[0]:
            print "*******************"
            print "Start Undo .."
            print "*******************"
            print "~~~~~~~~~~~~~~~~~"
            print "[Service Classes]"
            print "~~~~~~~~~~~~~~~~~"

            undoServiceClasses()

            print "*******************"
            print "~~~~~~~~~~~~~~"
            print "[Work Classes]"
            print "~~~~~~~~~~~~~~"

            undoWorkClasses()

            print "*******************"
            print "Saves WorkSpace .."

            AdminConfig.save()

            print "*******************"
            print "Finished."
            print "*******************"
    #    else:
    #        print "Non-Applicable Installation:  Current Count = %s." % theCount[1]
    #        print "Skip Undo."
    except Exception, e:
        print "Exception: %s" % e
        print sys.exc_info()
        print "Exit Script."
