#=======================================================================================================================================================
#(C) Copyright IBM Corp. 2005 - All Rights Reserved.
#DISCLAIMER:
#The following source code is sample code created by IBM Corporation.
#This sample code is provided to you solely for the purpose of assisting you
#in the  use of  the product. The code is provided 'AS IS', without warranty or
#condition of any kind. IBM shall not be liable for any damages arising out of your
#use of the sample code, even if IBM has been advised of the possibility of
#such damages.
#=======================================================================================================================================================

#=======================================================================================================================================================
# This file contains a series of operations to help setting up the Long Running Scheduler (LRS)
# or Long Running Execution Environment (LREE).
#
# Change History
# --------------
# 11-02-05 initial version created
#=======================================================================================================================================================

import sys
lineSeparator  = java.lang.System.getProperty("line.separator")
fileSeparator  = java.lang.System.getProperty("file.separator")
wasInstallRoot = java.lang.System.getProperty("was.install.root")



def printHelp():
  print """

  For General Help:
      >> ./wsadmin.sh -lang jython -f setupLongRunning.py

  For Type Specific Help:
      >> ./wsadmin.sh -lang jython -f setupLongRunning.py <type> --help
 
  Supported Types:
    lrs [options]
    lree [options]

  """
    
    
def printLRSHelp():
  print """
  
      lrs [options]
       
      Setup the Long Running Scheduler (LRS) that includes:
        - create dynamic cluster for LRS
        - set dynamic cluster operational mode
        - set max number of cluster instances to 1
        - enable startup beans for cluster instances
        - setup virtual hosts for cluster instances
        - configure long-running scheduler
        - deploy long-running scheduler application
       
       
      options:

        --ngName         the node group to be set up LRS
 
        --dcName         dynamic cluster where the LRS application
                         to be deployed

        --dcMode         optional dynamic cluster operational mode
                           automatic
                           manual
                           supervised (default)
 
        --appName        optional application name
                         default is LongRunningScheduler
 
        --schema         optional database schema name for the LRS database
                         default is LRSSCHEMA

        --jndiName       data source where jobs will be stored 

        --aliasName      alias to a user name and password used to access
                         the data source
 
      Example:
        >> wsadmin.sh -lang jython -f setupLongRunning.py lrs --ngName SchedulerNG  --dcName Scheduler --jndiName jdbc/lrsched
       
  """



def printLREEHelp():
  print """
  
      lree [options]
       
      Setup the Long Running Execution Environment (LREE) that includes:
        - create dynamic clusters for LREE
        - set dynamic cluster operational mode
        - enable startup beans for cluster instances
        - setup virtual hosts for cluster instances
        - deploy LREE application

       
      options:

        --ngName         the node group to be set up LREE
 
        --dcName         dynamic cluster where the LREE application 
                         to be deployed

        --dcMode         optional dynamic cluster operational mode
                           automatic
                           manual
                           supervised (default)
 
        --appName        optional application name
                         default is LREE
 
        --dbType         database type which supported by the ejbdeploy command
                         following are some possible values:
                           DB2UDB_V81
                           DB2UDB_V82
                           DB2EXPRESS_V81
                           DB2EXPRESS_V82
                           CLOUDSCAPE_V5
                           DB2UDBOS390_V7
                           DB2UDBOS390_V8
                           ORACLE_V8
                           ORACLE_V9I
                           ORACLE_V10G
                                             
        --schema         optional database schema name for the LREE database
                         default is LREESCHEMA

        --jndiName       data source where jobs status and result 
                         will be stored 

      Example:
        >> wsadmin.sh -lang jython -f setupLongRunning.py lree --ngName NodeGroup1 --dcName EndPoint1 --jndiName jdbc/lree --dbType DB2UDB_V81
 
  """



def processArguments():
  global  ngName, dcName, jndiName, appName, aliasName, dbType, schema, dcMode

  ngName    = ""
  dcName    = ""
  jndiName  = ""
  appName   = ""
  aliasName = ""
  dbType    = ""
  schema    = ""
  dcMode    = ""

  for i in range(1,len(sys.argv)):
    arg = sys.argv[i]
    if(arg == "--ngName"):
      ngName = sys.argv[i + 1]
      print "INFO: Node Group Name        = " + ngName
    elif(arg == "--dcName"):
      dcName = sys.argv[i + 1]
      print "INFO: Dynamic Cluster Name   = " + dcName
    elif(arg == "--dcMode"):
      dcMode = sys.argv[i + 1]
      print "INFO: Dynamic Cluster Mode   = " + dcMode
    elif(arg == "--appName"):
      appName = sys.argv[i + 1]
      print "INFO: Application Name       = " + appName
    elif(arg == "--dbType"):
      dbType = sys.argv[i + 1]
      print "INFO: Database Type          = " + dbType
    elif(arg == "--schema"):
      schema = sys.argv[i + 1]
      print "INFO: Database Schema name   = " + schema
    elif(arg == "--jndiName"):
      jndiName = sys.argv[i + 1]
      print "INFO: JNDI Name              = " + jndiName
    elif(arg == "--aliasName"):
      aliasName = sys.argv[i + 1]
      print "INFO: Data Source Alias Name = " + aliasName
    elif(arg.find("--")==0):
      print "WARNING: Unknown option: " + arg
  print ""



def isJNDINameDefinedInCell():

  # Get all JDBC providers defined at cell level
  cid       = AdminConfig.getid ("/Cell:" + AdminControl.getCell() + "/")
  jdbcps    = AdminConfig.list("JDBCProvider", cid)
  jdbcpList = jdbcps.split(lineSeparator)

  for jdbcp in jdbcpList:

    # Get all data sources of each JDBC provider
    dss    = AdminConfig.list("DataSource", jdbcp)
    dsList = dss.split(lineSeparator)
    
    for ds in dsList:

      # Check the JNDI name
      if ds != "":
         if AdminConfig.showAttribute(ds, "jndiName") == jndiName:
            return 1
  return 0



def getEndPointPort(nodeName, serverName):

  # Get all the server entries defined at node
  nid    = AdminConfig.getid("/Node:" + nodeName + "/")
  ses    = AdminConfig.list("ServerEntry", nid)
  seList = ses.split(lineSeparator)

  for se in seList:

    seName = AdminConfig.showAttribute(se, "serverName")
    if seName == serverName:

       # Found the server entry and get all end points
       neps = AdminConfig.list("NamedEndPoint", se)
       nepList = neps.split(lineSeparator)
       
       for nep in nepList:
       
         # Get the end point name
         epName = AdminConfig.showAttribute(nep, "endPointName")
         if epName == "WC_defaulthost":
         
            # Found the end point and get its port
            ep   = AdminConfig.showAttribute(nep, "endPoint")
            port = AdminConfig.showAttribute(ep, "port")

            return port

  return 0
  
 
  
def createDC(maxInstances):

  # Check the dynamic cluster
  dcs    = AdminConfig.list("DynamicCluster")
  dcList = dcs.split(lineSeparator)
  found  = "false"
  for dc in dcList:
    if dc.split("(")[0] == dcName:
       found = "true"
       
  if found=="true":
     print "INFO: Dynamic cluster " + dcName + " already exists"
     
     # Make sure that the node group matched
     dc = AdminConfig.getid("/ServerCluster:" + dcName +"/")
     ngn = AdminConfig.showAttribute(dc, "nodeGroupName")
     if ngn != ngName:
        print "ERROR: Dynamic cluster " + dcName + " is not belongs to node group " + ngName
        sys.exit(1)

  else:

     # Dynamic Cluster is not defined, so create it
     dccmbean = AdminControl.queryNames("WebSphere:*,process=dmgr,type=DynamicClusterConfigManager")
     AdminControl.invoke(dccmbean, "createDynamicCluster", ngName + " " + dcName + " \"\" \"\"")
     AdminConfig.save()
     print "INFO: Created dynamic cluster " + dcName

  # Set dynamic cluster operational mode and max number of cluster instances
  dcs    = AdminConfig.list("DynamicCluster")
  dcList = dcs.split(lineSeparator)
  for dc in dcList:
    if dc.split("(")[0] == dcName:
       AdminConfig.modify(dc, [["operationalMode", dcMode]])
       print "INFO: Set dynamic cluster " + dcName + " mode to " + dcMode
       if (maxInstances > 0):
          AdminConfig.modify(dc, [["maxInstances", "1"]])
          print "INFO: Set dynamic cluster " + dcName + " maxInstances to 1"
       break
     


def enableStartupBeansAndSetVirtualHost():

  # Get all cluster members of the dynamic cluster
  dc      = AdminConfig.getid("/ServerCluster:" + dcName +"/")
  members = AdminConfig.list("ClusterMember", dc)
  memList = members.split(lineSeparator)

  for mem in memList:

    # Enable the StartupBeansService
    memName = AdminConfig.showAttribute(mem, "memberName")
    server  = AdminConfig.getid("/Server:" + memName + "/")
    sbs     = AdminConfig.list("StartupBeansService", server)
    AdminConfig.modify(sbs, [["enable", "true"]])
    print "INFO: Enabled StartupBeansService on " + memName

    # Get the end point WC_defaulthost port
    nodeName        = AdminConfig.showAttribute(mem, "nodeName")
    defaultHostPort = getEndPointPort(nodeName, memName)
    if defaultHostPort > 0:
    
       # Determine the port is defined or not
       defaultHost     = AdminConfig.getid("/VirtualHost:default_host/")
       has    = AdminConfig.list("HostAlias", defaultHost)
       haList = has.split(lineSeparator)
       found = "false"
       for ha in haList:
           haPort = AdminConfig.showAttribute(ha, "port")
           haHost = AdminConfig.showAttribute(ha, "hostname")
           if haPort == defaultHostPort and haHost == "*":
              found = "true"
              
       if found == "true":
          print "INFO: WC_defaulthost port " + str(defaultHostPort) + " of " + memName + " is already added to virtual host aliases"
       else:
          AdminConfig.create("HostAlias", defaultHost, [["port", str(defaultHostPort)], ["hostname", "*"]])
          print "INFO: Added WC_defaulthost port " + str(defaultHostPort) + " of " + memName + " to virtual host aliases"

    else:
       print "WARNING: Failed to determine the port of " + memName



def configLRS():

  AdminTask.modifyLongRunningSchedulerAttribute("[-name datasourceJNDIName -value " + jndiName + "]")

  if aliasName != "":
     AdminTask.modifyLongRunningSchedulerAttribute("[-name datasourceAlias -value " + aliasName + "]")

  p = AdminTask.listLongRunningSchedulerProperties()
  if p.find("com.ibm.websphere.longrun.databaseSchemaName") < 0:
     AdminTask.createLongRunningSchedulerProperty("[-name com.ibm.websphere.longrun.databaseSchemaName -value " + schema + "]")
  else:
     AdminTask.modifyLongRunningSchedulerProperty("[-name com.ibm.websphere.longrun.databaseSchemaName -value " + schema + "]")

  print "INFO: Configured Long Running Scheduler"



def installLRSApp():

  server   = "WebSphere:cell=" + AdminControl.getCell() + ",cluster=" + dcName
  appPath  = wasInstallRoot + fileSeparator + "installableApps" + fileSeparator + "LongRunningScheduler.ear"

  mapWebModToVH   = [
                      [ "LongRunningJobSchedulerWebSvcRouter", "LRSWebSvc.war,WEB-INF/web.xml", "default_host" ]
                    ]
                  
  mapModToServers = [
                      [ "LongRunningJobSchedulerEJBs",         "LongRunningJobSchedulerEJBs.jar,META-INF/ejb-jar.xml",    server ],
                      [ "LongRunningJobSchedulerWebSvcRouter", "LRSWebSvc.war,WEB-INF/web.xml", server ]
                    ]
                        
  options = [
              "-usedefaultbindings",
              "-useMetaDataFromBinary",
              "-appname",                appName,
              "-MapWebModToVH",          mapWebModToVH, 
              "-MapModulesToServers",    mapModToServers
            ]
            
  print "INFO: Installing application " + appPath
  AdminApp.install(appPath, options)



def installLREEApp():

  server   = "WebSphere:cell=" + AdminControl.getCell() + ",cluster=" + dcName
  appPath  = wasInstallRoot + fileSeparator + "installableApps" + fileSeparator + "LREE.ear"

  mapWebModToVH     = [
                        [ "EndpointWebService", "EndpointWebService.war,WEB-INF/web.xml", "default_host" ]
                      ]
                  
  mapModToServers   = [
                        [ "BatchJobExecutionEnvironmentEJBs", "BatchJobExecutionEnvironmentEJBs.jar,META-INF/ejb-jar.xml",  server ],
                        [ "EndpointWebService",               "EndpointWebService.war,WEB-INF/web.xml",                     server ]
                      ]

  dsFor20EJBModules = [
                        [ "BatchJobExecutionEnvironmentEJBs", "BatchJobExecutionEnvironmentEJBs.jar,META-INF/ejb-jar.xml", jndiName ]
                      ]
                      
  dsFor20CMPBeans   = [
                        [ "BatchJobExecutionEnvironmentEJBs", "BjeeLogMessages",      "BatchJobExecutionEnvironmentEJBs.jar,META-INF/ejb-jar.xml", jndiName],
                        [ "BatchJobExecutionEnvironmentEJBs", "LocalJobStatus",       "BatchJobExecutionEnvironmentEJBs.jar,META-INF/ejb-jar.xml", jndiName],
                        [ "BatchJobExecutionEnvironmentEJBs", "Counter",              "BatchJobExecutionEnvironmentEJBs.jar,META-INF/ejb-jar.xml", jndiName],
                        [ "BatchJobExecutionEnvironmentEJBs", "AbstractResource",     "BatchJobExecutionEnvironmentEJBs.jar,META-INF/ejb-jar.xml", jndiName],
                        [ "BatchJobExecutionEnvironmentEJBs", "CheckpointRepository", "BatchJobExecutionEnvironmentEJBs.jar,META-INF/ejb-jar.xml", jndiName],
                        [ "BatchJobExecutionEnvironmentEJBs", "JobStepStatus",        "BatchJobExecutionEnvironmentEJBs.jar,META-INF/ejb-jar.xml", jndiName],
                        [ "BatchJobExecutionEnvironmentEJBs", "JobResults",           "BatchJobExecutionEnvironmentEJBs.jar,META-INF/ejb-jar.xml", jndiName]
                      ]
                        
  options = [
              "-usedefaultbindings",
              "-useMetaDataFromBinary",
              "-deployejb",
              "-appname",                   appName,
              "-MapWebModToVH",             mapWebModToVH,
              "-MapModulesToServers",       mapModToServers,
              "-DataSourceFor20EJBModules", dsFor20EJBModules,
              "-DataSourceFor20CMPBeans",   dsFor20CMPBeans,
              "-deployejb.dbtype",          dbType,
              "-deployejb.dbschema",        schema
            ]
            
  print "INFO: Installing application " + appPath
  AdminApp.install(appPath, options)
  


def save():

  print "INFO: Saving configuration..."
  AdminConfig.save()
  dmgrbean = AdminControl.queryNames("type=DeploymentManager,*")
  AdminControl.invoke(dmgrbean, "syncActiveNodes", "true")
  print "INFO: Configuration was saved and synchronized to the active nodes"
  


def setupLRS():

  global dcMode, schema, appName

  # Check the arguments
  if ngName == "":
     print "ERROR: Must specify node group name by --ngName"
     printLRSHelp()
     return 1

  if dcName == "":
     print "ERROR: Must specify dynamic cluster name by --dcName"
     printLRSHelp()
     return 1

  if dcMode == "":
     dcMode = "supervised"
  else:
     if dcMode != "supervised" and dcMode != "automatic" and dcMode != "manual":
        print "ERROR: Invalid dynamic cluster mode - " + dcMode
        printLRSHelp()
        return 1

  if schema == "":
     schema = "LRSSCHEMA"
     
  if appName == "":
     appName = "LongRunningScheduler"
     
  if jndiName == "":
     print "ERROR: Must specify JNDI name by --jndiName"
     printLRSHelp()
     return 1
  else:
     if not isJNDINameDefinedInCell():
        print "WARNING: JNDI name " + jndiName + " is not defined at cell level"
        
  # Make sure that the node group exists
  ngs    = AdminConfig.list("NodeGroup")
  ngList = ngs.split(lineSeparator)
  found  = "false"
  for ng in ngList:
    if ng.split("(")[0] == ngName:
       found = "true"
  if found=="false":
     print "ERROR: Node group " + ngName + " does not exist. Please provide a valid node group name."
     return 1

  # Make sure that the application name is not used
  apps    = AdminApp.list()
  appList = apps.split(lineSeparator)
  for app in appList:
    if app == appName:
       print "ERROR: Application " + appName + " already exists. Please provide a new application name."
       return 1

  print "INFO: Setting up Long Running Scheduler (LRS)..."
  createDC(1)
  enableStartupBeansAndSetVirtualHost()
  configLRS()
  installLRSApp()
  save()
  print "INFO: LRS was successfully configured"

  return 0
  


def setupLREE():

  global dcMode, schema, appName
  
  # Check the arguments
  if ngName == "":
     print "ERROR: Must specify the node group name by --ngName"
     printLREEHelp()
     return 1

  if dcName == "":
     print "ERROR: Must specify dynamic cluster name by --dcName"
     printLREEHelp()
     return 1

  if dcMode == "":
     dcMode = "supervised"
  else:
     if dcMode != "supervised" and dcMode != "automatic" and dcMode != "manual":
        print "ERROR: Invalid dynamic cluster mode - " + dcMode
        printLREEHelp()
        return 1

  if dbType == "":
     print "ERROR: Must specify the database type by --dbType"
     printLREEHelp()
     return 1

  if schema == "":
     schema = "LREESCHEMA"
     
  if appName == "":
     appName = "LREE"

  if jndiName == "":
     print "ERROR: Must specify JNDI name by --jndiName"
     printLREEHelp()
     return 1
  else:
     if not isJNDINameDefinedInCell():
         print "WARNING: JNDI name " + jndiName + " is not defined at cell level"

  # Make sure that the node group exists
  ngs    = AdminConfig.list("NodeGroup")
  ngList = ngs.split(lineSeparator)
  found  = "false"
  for ng in ngList:
    if ng.split("(")[0] == ngName:
       found = "true"
  if found=="false":
     print "ERROR: Node group " + ngName + " does not exist. Please provide a valid node group name."
     return 1

  # Make sure the application does not exist
  apps    = AdminApp.list()
  appList = apps.split(lineSeparator)
  for app in appList:
    if app == appName:
       print "ERROR: Application " + appName + " already exists. Please provide a new application name."
       return 1
                        
  print "INFO: Setting up Long Running Execution Environment (LREE)..."
  createDC(0)
  enableStartupBeansAndSetVirtualHost()
  installLREEApp()
  save()
  print "INFO: LREE was successfully configured"

  return 0
  


#=======================================================================================================================================================
#
# Main execution logic:
#
#=======================================================================================================================================================

if len(sys.argv) > 0:
   option = sys.argv[0]
   help = 0
   if len(sys.argv) > 1:
      if sys.argv[1] == "--help":
         help = 1
   if not help:
      processArguments()
   if option == "lrs":
      if help == 1:
         printLRSHelp()
      else:
         sys.exit(setupLRS())
   elif option == "lree":
      if help == 1:
         printLREEHelp()
      else:
         sys.exit(setupLREE())
   else:
      printHelp()
else:
   printHelp()
