-- This script creates Long Running Execution Environment tables on a Informix instance.
-- The default name of the database is LRSCHED, default schema name is
-- LRSSSCHEMA and default tablespace is rootdbs. All these defaults
-- can be replaced with different names if needed.
-- The following command can be issued from Informix command line processor.
-- To run this script type at cmd-line in <informix_install>/bin
-- dbaccess - CreateLrschedTablesInformix.sql



CONNECT TO "LRSCHED";

BEGIN WORK;

---------------------------------------
--DDL statements for schema: "LRSSCHEMA
---------------------------------------
CREATE SCHEMA AUTHORIZATION LRSSCHEMA;



------------------------------------------------------------
--DDL statements for table: "LRSSCHEMA.GLOBALJOBIDASSIGNMENT
------------------------------------------------------------
CREATE TABLE LRSSCHEMA.GLOBALJOBIDASSIGNMENT (

		  JOBSCHEDULERNAME VARCHAR(250) NOT NULL , 

		  JOBNUMBER INTEGER NOT NULL
                  )
                  IN rootdbs;  

-------------------------------------------------------------
-- DDL Statements for primary key on Table: "LRSSCHEMA.GLOBALJOBIDASSIGNMENT"
-----------------------------------------------------------
ALTER TABLE LRSSCHEMA.GLOBALJOBIDASSIGNMENT
      ADD CONSTRAINT PRIMARY KEY(JOBSCHEDULERNAME,JOBNUMBER) CONSTRAINT PK_GLOBALJOBIDASS2;




---------------------------------------------------
--DDL statements for index: "LRSSCHEMA.JOBNUM_INDEX
---------------------------------------------------
CREATE INDEX LRSSCHEMA.JOBNUM_INDX 
          ON GLOBALJOBIDASSIGNMENT (JOBNUMBER ASC);



--------------------------------------------------
--DDL statements for table: "LRSSCHEMA.LOGMESSAGES
--------------------------------------------------		
CREATE TABLE LRSSCHEMA.LOGMESSAGES  (

		  JOBID VARCHAR(130) NOT NULL , 

		  MSGSEQ INTEGER NOT NULL , 

		  MSGTXT VARCHAR(250)  
                  
                  )
                  IN rootdbs;
-------------------------------------------------------------
-- DDL Statements for primary key on Table: "LRSSCHEMA.LOGMESSAGES"
-----------------------------------------------------------
ALTER TABLE LRSSCHEMA.LOGMESSAGES
      ADD CONSTRAINT PRIMARY KEY(JOBID,MSGSEQ) CONSTRAINT PK_LOGMESSAGES;




------------------------------------------------
--DDL statements for table: "LRSSCHEMA.JOBSTATUS
------------------------------------------------	
CREATE TABLE LRSSCHEMA.JOBSTATUS  (

		  JOBID VARCHAR(130) NOT NULL , 

		  STATUS INTEGER NOT NULL , 

		  RC INTEGER NOT NULL , 

		  UPDATECNT INTEGER NOT NULL , 

		  LASTUPDATE VARCHAR(250) , 

		  SUSPENDEDUNTIL VARCHAR(250) , 

		  SCHEDULEROWNS SMALLINT NOT NULL , 

		  OWNINGBJEE VARCHAR(250) , 

		  TYPE VARCHAR(16),

		  SUBMITTER VARCHAR(32),

		  NODE VARCHAR(250),

		  APPSERVER VARCHAR(250),

		  STATUSTXT VARCHAR(250),

                  STARTTIME VARCHAR(250),

                  REQUESTID VARCHAR(250)
                  )
                  IN rootdbs;  

-------------------------------------------------------------
-- DDL Statements for primary key on Table: "LRSSCHEMA.JOBSTATUS"
-----------------------------------------------------------
ALTER TABLE LRSSCHEMA.JOBSTATUS
      ADD CONSTRAINT PRIMARY KEY(JOBID) CONSTRAINT PK_JOBSTATUS;




-------------------------------------------------------
--DDL statements for table: "LRSSCHEMA.SCHEDULERCOUNTER
-------------------------------------------------------
CREATE TABLE LRSSCHEMA.SCHEDULERCOUNTER  (

		  CNTNAME VARCHAR(250) NOT NULL , 

		  CNT INTEGER NOT NULL
                  )
                  IN rootdbs;

-------------------------------------------------------------
-- DDL Statements for primary key on Table: "LRSSCHEMA.SCHEDULERCOUNTER"
-----------------------------------------------------------
ALTER TABLE LRSSCHEMA.SCHEDULERCOUNTER
      ADD CONSTRAINT PRIMARY KEY(CNTNAME) CONSTRAINT PK_SCHEDULERCOUNTS;




-----------------------------------------------------
--DDL ststements for table: "LRSSCHEMA.XJCLREPOSITORY
-----------------------------------------------------
CREATE TABLE LRSSCHEMA.XJCLREPOSITORY (

		  JOBID VARCHAR(130) NOT NULL , 

		  SEQ INTEGER NOT NULL , 

		  TXT VARCHAR(250)
                  )
                  IN rootdbs;

-------------------------------------------------------------
-- DDL Statements for primary key on Table: "LRSSCHEMA.XJCLREPOSITORY"
-----------------------------------------------------------
ALTER TABLE LRSSCHEMA.XJCLREPOSITORY
      ADD CONSTRAINT PRIMARY KEY(JOBID,SEQ) CONSTRAINT PK_XJCLRESPOSITORY;



 
----------------------------------------------------
--DDL statements for table: "LRSSCHEMA.JOBREPOSITORY
----------------------------------------------------
CREATE TABLE LRSSCHEMA.JOBREPOSITORY  (

		  JOBNAME VARCHAR(250) NOT NULL , 

		  SEQ INTEGER NOT NULL , 

		  TXT VARCHAR(250)
                  )
                  IN rootdbs; 

-------------------------------------------------------------
-- DDL Statements for primary key on Table: "LRSSCHEMA.JOBREPOSITORY"
-----------------------------------------------------------
ALTER TABLE LRSSCHEMA.JOBREPOSITORY
      ADD CONSTRAINT PRIMARY KEY(JOBNAME,SEQ) CONSTRAINT PK_JOBRESPOSITORYY;



--------------------------------------------------------------------
--DDL statements for table "LRSSCHEMA.RECURRINGREQUEST"
--------------------------------------------------------------------
CREATE TABLE LRSSCHEMA.RECURRINGREQUEST  (

		  REQUESTID VARCHAR(250) NOT NULL , 

		  JOBNAME VARCHAR(250) NOT NULL , 
                  
                  STARTTIME VARCHAR(250) NOT NULL,
 
                  INTERVAL VARCHAR(250) NOT NULL,
          
                  SUBMITTER VARCHAR(32),
      
                  STATUS INTEGER,

                  FIELD1 INTEGER,

                  FIELD2 VARCHAR(250)

		  )
                  IN rootdbs; 

-----------------------------------------------------------------------
-- DDL Statements for primary key on Table: "LRSSCHEMA.RECURRINGREQUEST"
-----------------------------------------------------------------------
ALTER TABLE LRSSCHEMA.RECURRINGREQUEST
      ADD CONSTRAINT PRIMARY KEY(REQUESTID) CONSTRAINT PK_RECURRINGREQUEST;
                 


COMMIT WORK;

