// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2004, 2005
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.

//parent: the node(in the data tree) that should be the parent of any children of this node
//purpose: the purpose of this tree (what centricity)
//elem: the XML element containing updated information
function handleXMLElement(purpose, elem, parent){

	var nodeId = elem.getAttribute(ATTR_ID);
	var node = nodeIndex[nodeId];
	if (!node)
		return;
		
	var type = elem.getAttribute(ATTR_TYPE);
	var dels = elem.getElementsByTagName(ELEMENT_DELETE);
	var adds = elem.getElementsByTagName(ELEMENT_ADD);
	var changes = elem.getElementsByTagName(ELEMENT_CHANGE);
	var changed;
    var mainNodeChanged = false;
    	
   if (parent == null){ //special case of the recursion - the root node
   	mainNodeChanged = updateNode(elem, purpose) || mainNodeChanged;   	
   }
    	
	if (elem.hasChildNodes()){	
        var currNode;
			for(var c = 0; c < elem.childNodes.length; c++){
				changed = false;
	            currNode = elem.childNodes.item(c);

			// We don't display the All entry on the treeLayout.
			if (currNode.getAttribute(ATTR_NAME) == "All")
				continue;
	            
	         if (currNode.nodeName == ELEMENT_ADD){
	            if (nodeExists(currNode))
						changed = updateNode(currNode, purpose);
					else {
						addNode(currNode, purpose, node);
						changed = true;
					}
				} else if (currNode.nodeName == ELEMENT_DELETE){
					delNode(currNode, purpose);
					changed = false; //we don't want to try to reset the innerHTML of a nonexistent node
					mainNodeChanged=true;
				} else if (currNode.nodeName == ELEMENT_CHANGE){
					changed = updateNode(currNode, purpose);
				}		 		
	    		
	        if (changed) {  //if this child changed, we need to set its innerHTML
	            childNode = nodeIndex[currNode.getAttribute(ATTR_ID)];
	            if (childNode){ //it could have been removed in the interim...
		            childNode.layer.innerHTML = childNode.getInnerHtml(purpose);
		            mainNodeChanged = true;
	            }
	        }
	     }
		
		//we have now finished adding/moving/updating this node's children, let's call each of its children recursively.		
        var children = elem.childNodes;
        var childElem;
        for (var i = 0; i < children.length; i++){
            childElem = children.item(i);
            mainNodeChanged=handleXMLElement(purpose, childElem, node) || mainNodeChanged;
        }      
    }      

	if (mainNodeChanged)
		node.layer.innerHTML = node.getInnerHtml();
		
	node.setPadding(node.layer);   
   return mainNodeChanged;
}   

function nodeExists(elem){
	return nodeIndex[elem.getAttribute(ATTR_ID)];	
}

function delNode(elem, purpose){
	var id = elem.getAttribute(ATTR_ID);
	
	if (nodeWithCurrentPopup && nodeWithCurrentPopup.id == id)
		hideProvisioningPopup();
	
	var type = elem.getAttribute(ATTR_TYPE);
	if (!id || !type)
		return false; 
	
	var genericHandler = genericHandlers[type];
	if (genericHandler)
		genericHandler.remove(elem);
	else{
		var parent;
		while (id){
			parent = nodeIndex[id].parent;
			delTreeNode(id);	
			id = null;
			
			if (parent && parent.childCount == 0 && parent.type == TYPE_LABEL){
				id = parent.id; //parent is just a label, and it has no more children, so it's not needed.
			} else if (parent && parent.childCount == 0){ //we know it decreased, and now has one...so we need to adjust the padding and get rid of the plus/minus icon.
				parent.layer.innerHTML = parent.getInnerHtml(purpose);
				parent.setPadding(parent.layer);
			}
		}
	}
	
	return true;
}

//if a node with this id exists anywhere in the tree, it is first removed, then this node is added.
function addNode(elem, purpose, parent){
	var name = elem.getAttribute(ATTR_NAME);
	//if (name == currentRootName) //this will break in cases where the rootname is duplicated (e.g. application centric view with a .war application name)
	//	return;
	var type = elem.getAttribute(ATTR_TYPE);	
	var id = elem.getAttribute(ATTR_ID);
	if (!type || !id) return false;
	if (id == parent.id)
		return;

	var genericHandler = genericHandlers[type];
	if (genericHandler)
		genericHandler.add(elem);
	else {
		delTreeNode(id);	 //if any node exists with this id already, remove it.	
		
		var populatorFunction = populatorFunctions[type];
		var iconFunction = iconFunctions[type];
		var innerHTMLFunction = innerHTMLFunctions[type];
		var linkFunction = linkFunctions[type];
		var contentFunction = contentFunctions[type];
		if (!populatorFunction || !iconFunction || !linkFunction || !innerHTMLFunction)
			return false;
			
		//parent finder functions accept (among other things) a node parameter, which represents the entity 
		//which is the logical parent of the child node being added.  The method then returns the actual node in the 
		//tree which is to be the parent of the created node.  For instance, when adding a Dynamic Cluster to a NodeGroup, the 
		//node group is the logical parent.  However, a special "Dynamic Clusters" node is the actual parent of the new node.
		var parentNode;
		if (parent){
			parentNode = findParentNode(elem, purpose, parent); 
		} 

		var fakeNode = new Object(); //use this to hold the data to pass into the various methods used in the call to addItem()
		populatorFunction(elem, purpose, fakeNode);
		//we use this link (return false;) because we want the text to be a link, but we don't want it to go anywhere. (just pop up a menu)
		var link = "";
		var iconInfo = iconFunction(fakeNode, purpose);
      var node = addItem(purpose, parentNode, iconInfo.url, iconInfo.alt, contentFunction(fakeNode, purpose), link, elem.getAttribute(ATTR_ID), innerHTMLFunction, type);
		populatorFunction(elem, purpose, node);
 		node.layer.innerHTML = node.getInnerHtml();
 
		if(parentNode && parentNode.childCount == 1)
			parentNode.setPadding(parentNode.layer);
	}
	
	document.getElementById("progress").style.display = "block"; 
	document.getElementById("progress").style.display = "none";
	
	return true;
}

function updateNode(elem, purpose){
	var id = elem.getAttribute(ATTR_ID);
	var type = elem.getAttribute(ATTR_TYPE);	
	var changed = false;
	if (!id || !type)
		return changed;

	var genericHandler = genericHandlers[type];
	if (genericHandler) // this shouldn't actually happen - the only things that have generic handlers are RootNames, and they don't change - they either get deleted or added.
		changed = genericHandler.update(elem);
	else {	
		var node = nodeIndex[id];
		var populatorFunction = populatorFunctions[type];
		var tmp;
		changed = populatorFunction(elem, purpose, node);
		
		tmp = contentFunctions[type](node, purpose);
		if (tmp && node.content != tmp) {
			node.content = tmp;
			changed = true;			
		}
		
		tmp = linkFunctions[type](node, purpose);
		if (tmp && tmp != node.link) {
			node.link = tmp;
			changed = true;			
		}
		
		tmp = iconFunctions[type](node, elem);
	    if (tmp && tmp != node.icon) {
	            node.icon = tmp.url;
	            node.icon_alt = tmp.alt;
	            changed = true;
	    }

		//tmp = node.getInnerHtml(purpose);
		//if (tmp) node.layer.innerHTML = tmp;	

		//node.setPadding(node.layer);
	}
	return changed;
}

function getNodeFromElem(elem){
	var id = elem.getAttribute(ATTR_ID);
	if (!id)
		return null;
	return nodeIndex[id];	
}

