# (C) Copyright IBM Corp. 2004,2005 - All Rights Reserved.
# DISCLAIMER:
# The following source code is sample code created by IBM Corporation.
# This sample code is provided to you solely for the purpose of assisting you
# in the  use of  the product. The code is provided 'AS IS', without warranty or
# condition of any kind. IBM shall not be liable for any damages arising out of your
# use of the sample code, even if IBM has been advised of the possibility of
# such damages.


# workclassoperations.py - creates, updates work classess for enterprise
# applications and generic server clusters. It can also, list rules in a 
# work class, list URIs in a work class, fetch default action, set default 
# action, add rule, delete rule, delete URIs, and delete work  classes for 
# the various policies and communication protocols. 
#
# Date Created: 04/28/2005
#

#
# 
import jarray
from java.lang import System
from com.ibm.ws.grid import AppTypeDeploymentTask
from com.ibm.ws.classify import Expression
from com.ibm.ws.xd.config.workclass.util import ApplicationUtil
from com.ibm.wsspi.classify import RuleParserException 

def setProtocolForWorkClass():
   if (myProtocolType == "SOAP"):
      protocolType = "SOAPWORKCLASS"
   elif (myProtocolType == "IIOP"):
      protocolType = "IIOPWORKCLASS"
   elif (myProtocolType == "JMS"):
      protocolType = "JMSWORKCLASS"
   else:
      protocolType = "HTTPWORKCLASS"
   return protocolType 

def displayRules(wcId):
     mAction=""
     mExpr=""
     rList = AdminConfig.showAttribute(wcId,"matchRules").replace("[","").replace("]","").split(" ")
     dAction = AdminConfig.showAttribute(wcId,"matchAction")
     #print "Len of rList is %i" %(len(rList))
     if (isGetDefaultAction == 1):
         print "Default Action: "+dAction
     else:
       for aRule in rList:
         try:
           mPriority=AdminConfig.showAttribute(aRule,"priority")
           mAction=AdminConfig.showAttribute(aRule,"matchAction")
           mExpr=AdminConfig.showAttribute(aRule,"matchExpression")
         except:
             print ""
         if (isListRule == 1):
             if (mExpr != "" and mAction != ""):
                print "Priority: "+mPriority+"; Expression: "+mExpr + "; Action: "+mAction
     	  
def findGenericServerCluster(anAction):
   gscNames=AdminConfig.list("GenericServerCluster").split("\n")
   for gscName in gscNames:
       gscName=gscName.rstrip()
       if (len(gscName) > 0 ):
          name=AdminConfig.showAttribute(gscName,"name")
          if (anAction == name):
             #print "matched Generic Server Cluster: " + name
             return name
   return ""  
   
def findProxyServer(anOdrName):
   psNames=AdminConfig.list("ProxyServer").split("\n")
   for psName in psNames:
       psName=psName.rstrip()
       if (len(psName) > 0 and psName.find(anOdrName) > -1):
             #print "ODR name found in: " + psName
             return psName
   return "" 
   
def findNode(anOdrNode):
   vhNames=AdminConfig.list("Node").split("\n")
   for vhName in vhNames:
       vhName=vhName.rstrip()
       if (len(vhName) > 0 ):
          name=AdminConfig.showAttribute(vhName,"name")
          if (anOdrNode == name):
             #print "matched ODR Node: " + name
             return name
   return "" 
   
def findVirtualHost(aVHost):
   vhNames=AdminConfig.list("VirtualHost").split("\n")
   for vhName in vhNames:
       vhName=vhName.rstrip()
       if (len(vhName) > 0 ):
          name=AdminConfig.showAttribute(vhName,"name")
          if (aVHost == name):
             #print "matched Virtual Host: " + name
             return name
   return "" 
 
def isMatchOverlapping(aMatch):
    #if (patternOverlaps == ""):
    patternOverlaps = ""
    
    if (myWorkClassType == "ARP" or myWorkClassType == "ASP"):    
        if (myWorkClassType == "ARP"):
            patternOverlaps = getAppRoutingWorkClassesURIs()
        elif (myWorkClassType == "ASP"):
            patternOverlaps = getAppSLAWorkClassesURIs()

        patternOverlaps = patternOverlaps.split(",")
        
        for pattern in patternOverlaps:
            if (pattern == aMatch+" ("+myWebModule+")"):
                return "true"
            
    if (myWorkClassType == "GRP" or myWorkClassType == "GSP"):
        if (myWorkClassType == "GRP"):
            patternOverlaps = getGSCWorkClassesURIs("routing")
        elif (myWorkClassType == "GSP"):
            patternOverlaps = getGSCWorkClassesURIs("sla")
        
        patternOverlaps = patternOverlaps.split(",")
        
        for pattern in patternOverlaps:
            if (pattern == aMatch):
                return "true"
    
    return "false"
 
      
def getAppRoutingWorkClassesURIs():
   wcNames=AdminConfig.list("WorkClass").split("\n")
   wcURIs = ""
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          appName = myAppName.split("-edition")[0]          
          if (wcName.find(appName+".ear") > 0 and wcName.find("odr") == -1 and wcName.find("deployments") == -1):
             wcmList = AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")

             for wcm in wcmList: 
                if (wcm == ""):
                   break                   
                wcmId = wcm
                wcmName = AdminConfig.showAttribute(wcmId, "moduleName")
                uris = AdminConfig.showAttribute(wcmId, "matchExpression")
                uris = uris.split(",")
                wcmName = wcmName.split(".")[0]                
                
                for uri in uris:
                   if (wcURIs == ""):
                       wcURIs = uri+" ("+wcmName+")"
                   else:
                       wcURIs = wcURIs+","+uri+" ("+wcmName+")" 
   
   return wcURIs 
  
def findAppRoutingWorkClass(aWCName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName = myAppName.split("-edition")[0]          
          if (aWCName == name and wcName.find(appName+".ear") > 0 and wcName.find("odr") == -1 and wcName.find("deployments") == -1):
             #appName=AdminConfig.showAttribute(wcName,"matchAction")
             appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
             if (appName.find(myAppName.split("-edition")[0]) > 0 or appName==""):
                return wcName
   return "" 

def findSystemAppRoutingWorkClass(aWCName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName = myAppName.split("-edition")[0]          
          if (aWCName == name and wcName.find(appName+".ear") > 0 and wcName.find("systemApps") > 0 and wcName.find("deployments") == -1):
                return wcName
   return "" 

def findSystemAppSLAWorkClass(aWCName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName = myAppName.split("-edition")[0]          
          if (aWCName == name and wcName.find(appName+".ear") > 0 and wcName.find("systemApps") > 0 and wcName.find("deployments") > 0):
                return wcName
   return "" 

def getAppSLAWorkClassesURIs():
   wcNames=AdminConfig.list("WorkClass").split("\n")
   wcURIs = ""
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          if (wcName.find("/"+myAppName+"/") > 0 and wcName.find("deployments") > 0):              
             wcmList = AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")
             for wcm in wcmList: 
                if (wcm == ""):
                   break                   
                wcmId = wcm
                wcmName = AdminConfig.showAttribute(wcmId, "moduleName")
                uris = AdminConfig.showAttribute(wcmId, "matchExpression")
                uris = uris.split(",")
                wcmName = wcmName.split(".")[0]
                
                for uri in uris:
                   if (wcURIs == ""):
                       wcURIs = uri+" ("+wcmName+")"
                   else:
                       wcURIs = wcURIs+","+uri+" ("+wcmName+")" 
   
   return wcURIs  
 
def findAppSLAWorkClass(aWCName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          if (aWCName == name and wcName.find("/"+myAppName+"/") > 0 and wcName.find("deployments") > 0):
              appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
              if (appName.find(myAppName) > 0):
                 #print "matched app SLA workclass: " + appName
                 return wcName
   return "" 


def getGSCWorkClassesURIs(aPolicyType):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   wcURIs = ""

   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 and wcName.find(aPolicyType) > -1 and wcName.find(myOdrName) > 0):
             wcmList = AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")
             for wcm in wcmList: 
                if (wcm == ""):
                   break              
                wcmId = wcm
                wcmName = AdminConfig.showAttribute(wcmId, "moduleName")
                uris = AdminConfig.showAttribute(wcmId, "matchExpression")
                uris = uris.split(",")
                wcmName = wcmName.split(".")[0]
                
                for uri in uris:
                   if (wcURIs == ""):
                       wcURIs = uri
                   else:
                       wcURIs = wcURIs+","+uri
   
   return wcURIs 

def findWorkClass(aWCName,aPolicyType):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 and wcName.find(aPolicyType) > -1 and wcName.find(myOdrName) > 0):
          name=AdminConfig.showAttribute(wcName,"name")
          if (aWCName == name):
             #print "matched WorkClass: " + name
             return wcName
   return ""  
  
def findTransactionClass(aTCName):
   tcNames=AdminConfig.list("TransactionClass").split("\n")
   for tcName in tcNames:
       tcName=tcName.rstrip()
       if (len(tcName) > 0 ):
          name=AdminConfig.showAttribute(tcName,"name")
          if (aTCName == name):
             #print "matched Transaction Class: " + name
             return name
   return ""

def findDeployment(applName):
   appNames=AdminConfig.list("Deployment").split("\n")
   for appName in appNames:
       appName=appName.rstrip()
       if (len(appName) > 0 ):
          if (appName.startswith(applName)):
             #print "matched application: " + appName
             return appName
   return ""

def findSystemApplication(aAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   earAppName=""
   if (aAppName.endswith(".ear")):
       earAppName=aAppName
   else:
       earAppName=aAppName+".ear"
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          if (name.startswith("Default_") and name.endswith("_WC")):
              appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
              if (appName.find(earAppName) > 0):
                 print "system application " + aAppName + " exists"
                 return aAppName
   return "" 
  
def checkForReservedWC():
    if (myWorkClassName.startswith("Default_") and myWorkClassName.endswith("_WC")):
        print ""
        print "You are not allowed to delete a default work class, goodbye."
        errorExit()

def checkForRequiredOptions():
  if (myWorkClassName == ""):
        print ""
        print "You must specify the work class name using the -wcname option."
  if (isAppWorkClasses == 1):
        if (myAppName == ""):
            print ""
            print "You have asked to process rules without specifying the enterprise application name."
            print "You must specify an enterprise application name using the -appname option."
        elif (isSystemApp == 1):
            anApp = findSystemApplication(myAppName)
        else:
            anApp=findDeployment(myAppName)
        if (anApp == ""):
            print "The application " + myAppName + " specified cannot be found in this WAS deployment"
            errorExit()      
  else:
        if (myOdrNode == "" or myOdrName == ""):
            print ""
            print "The rule operations for generic server cluster require the ODR node and name."
            print "You must use both -odrname and -odrnode to specify the ODR and ODR node respectively."
        anOdrNode=findNode(myOdrNode)
        if (anOdrNode != ""):
            anOdr=findProxyServer(myOdrName)
            if (anOdr == ""):
                print ""
                print "The ODR name " +myOdrName + " does not exist on this server."
        else:
            print ""
            print "The ODR node " +myOdrNode + " does not exist on this server."
            
def processRulesForWorkClass():
    checkForRequiredOptions()
    if (isAppWorkClasses == 1):
        if (myWorkClassType == "ARP" or myWorkClassType == "APP"):
           wcid =findAppRoutingWorkClass(myWorkClassName)
           if (wcid != ""):
              if (isListRule == 1):
                 print ""
                 print "Printing routing rules for work class: " +myWorkClassName
              elif (isGetDefaultAction == 1):
                 print ""
                 print "Printing routing default action for work class: " +myWorkClassName
              displayRules(wcid)
        if (isPerformBoth == 1 or myWorkClassType == "ASP"):
            wcid = findAppSLAWorkClass(myWorkClassName)
            if (wcid != ""):
                print ""
                if (isListRule == 1):
                    print "Printing SLA rules for work class: " +myWorkClassName
                elif (isGetDefaultAction == 1):
                    print "Printing SLA default action for work class: " +myWorkClassName
                displayRules(wcid)
    else:
        if (myWorkClassType == "GRP"):
            wcid = findWorkClass(myWorkClassName,"routing")
            if (wcid != ""):
               print ""
               if (isListRule == 1):
                     print "Printing routing rules for work class: " +myWorkClassName
               elif (isGetDefaultAction == 1):
                     print "Printing routing default action for work class: " +myWorkClassName
               displayRules(wcid)
        if (isPerformBoth == 1 or myWorkClassType == "GSP"):
            wcid = findWorkClass(myWorkClassName,"sla")
            if (wcid != ""):
               print ""
               if (isListRule == 1):
                    print "Printing SLA rules for work class: " +myWorkClassName
               elif (isGetDefaultAction == 1):
                    print "Printing SLA default action for work class: " +myWorkClassName
               displayRules(wcid)

def removeRuleFromWorkClass(anId):
     rulePriority=-1
     ruleList = AdminConfig.showAttribute(anId,"matchRules").replace("[","").replace("]","").split(" ")
     if (isAppWorkClasses == 1):
         wctype = "enterprise application"
     else:
         wctype = "generic server cluster"
     if (len(ruleList)> 0):
         #print "The number of rules in this work class is %d"  %(len(ruleList))
         for ar in ruleList:
             if (len(ar) > 0):
                rulePriority = AdminConfig.showAttribute(ar,"priority")
                if (rulePriority == myDeleteRulePriority):
                   AdminConfig.remove(ar)   
                   print "Rule with priority " + myDeleteRulePriority + " has been deleted from "+wctype+" work class " +myWorkClassName    
                   break 
                                    
def deleteWorkClassRule(): 
  checkForRequiredOptions() 
  if (isAppWorkClasses == 1):
        wcid =findAppRoutingWorkClass(myWorkClassName)
        if (wcid != ""):
              removeRuleFromWorkClass(wcid)
        if (isPerformBoth == 1):
            wcid = findAppSLAWorkClass(myWorkClassName)
            if (wcid != ""):
                removeRuleFromWorkClass(wcid) 
  else:
      wcid = findWorkClass(myWorkClassName,"routing")
      if (wcid != ""):
           removeRuleFromWorkClass(wcid)
      if (isPerformBoth == 1):
           wcid = findWorkClass(myWorkClassName,"sla")
           if (wcid != ""):
               removeRuleFromWorkClass(wcid)
  AdminConfig.save()
  
##
# Creates a WorkClass with user rules 
# for GSC Routing policy
##
def createGSCRoutingWorkClass():
   vhostName=findVirtualHost(myVirtualHost)
   odrNode=findNode(myOdrNode)
   odrName=findProxyServer(myOdrName)
   if (vhostName!="" and odrNode != "" and odrName != ""):
       if (myDefaultAction != ""):
           dParts = myDefaultAction.split("?")
           actionType = dParts[0]
           actionValue = ""
           try:
              actionValue = dParts[1]
           except IndexError:
              if (actionValue == "" and actionType.startswith("permit")):
                  print "The generic server cluster must be specified with -setdefaultaction option and a valid generic server cluster name."
              else:
                  print "You have specified a routing actiontype of either '"+actionType +"' without the appropriate action."
              errorExit()  
           if (actionType.startswith("permit")):
              gscName=findGenericServerCluster(actionValue) 
              if (gscName == ""):             
                 print "The generic server cluster specified is not valid."
                 errorExit() 
              #print "Task Command Input: " + taskCmdArgs
           taskCmdArgs =  "[-odrnode "+ myOdrNode +" -odrname "+ myOdrName+" -wcname "+myWorkClassName+" -type "+myProtocolType+" -actiontype "+actionType+" -action "+actionValue+" -vhost "+myVirtualHost+"]"
           if (isModifyDefaultAction ==1):
                 routingWorkClass = findWorkClass(myWorkClassName,"routing")
                 if (routingWorkClass != ""):
                     AdminConfig.modify(routingWorkClass,[["matchAction", myVirtualHost+":"+actionType+":"+actionValue]])
           else:
                 AdminTask.createRoutingPolicyWorkClass(taskCmdArgs)
       routingWorkClass=findWorkClass(myWorkClassName,"routing")
       if (routingWorkClass != ""):
          for k in range(0,len(myRule),1):
            if (myRule[k] != None):
               rParts = myRule[k].split("?")
               priority = rParts[0]
               expr = rParts[1]
               actionType = rParts[2]
               validateExpression(expr)
               try:
                  actionValue = rParts[3]
               except IndexError:
                  if (actionType.startswith("permit")):
                     print "The generic server cluster must be specified with -addrule option. The action argument must be the name of the generic server cluster."
                  else:
                      print "The reject code or redirect URL must be specified with the action type of reject or redirect respectively."
                  errorExit()
               if (actionType.startswith("permit")):
                   gscName=findGenericServerCluster(actionValue)
                   if (gscName == ""):             
                       print "The generic server cluster '"+actionValue +"' specified is not valid."
                       errorExit()
               print "priority = "+priority+ " expr="+expr+ " actionType="+actionType + " action="+actionValue
               wcRuleAttributes = [["matchAction",actionType+":"+actionValue],
                                   ["matchExpression",expr],
                                   ["priority",priority]]
               routingRules=AdminConfig.create("MatchRule",routingWorkClass,wcRuleAttributes)
               #print "the len size of rules is %i" %(len(myRule))
          if (myURIS != ""):
             uriExpression = ""
             for uri in myURIS.split("?"):
                if (uriExpression == ""):
                   uriExpression = uri
                else:
                   uriExpression = uriExpression + "," + uri
          else: 
            uriExpression = "" 
          wcModuleAttributes = [ ["moduleName",myOdrName],
                               ["matchExpression",uriExpression],
                               ["id",myWorkClassName+":!:"+myOdrName+":!:"+myOdrNode]]
          routingWorkClassModule=AdminConfig.create("WorkClassModule",routingWorkClass,wcModuleAttributes)
   else:
      if (vhostName == ""):
         errMsg = " virtual host."
      elif (odrNode == ""):
         errMsg = " ODR node."
      elif (odrName == ""):
         errMsg = " ODR."
      elif (vhostName == "" and odrNode == ""):
         errMsg = " the Virtual host and ODR node."
      elif (odrNode == "" and odrName == ""):
         errMsg = " ODR node and ODR name."
      else:
         errMsg = "check command arguments and re-issue command."
      print "The specified arguments are not found in this WAS deployment: "+errMsg
      errorExit()
      
##
# Creates a WorkClass with user defined rules 
# for SLA policy
##
def createGSCSLAWorkClass():
   vhostName=findVirtualHost(myVirtualHost)
   odrNode=findNode(myOdrNode)
   odrName=findProxyServer(myOdrName)
   if (vhostName!="" and odrNode != "" and odrName != ""):
       if (myDefaultAction != ""):
           dParts = myDefaultAction.split("?")
           actionType = dParts[0]
           try:
              actionValue = dParts[1]
           except IndexError:
              print 
              #errorExit()  
           taskCmdArgs =  "[-odrnode "+ myOdrNode +" -odrname "+ myOdrName+" -wcname "+myWorkClassName+" -type "+myProtocolType+" -tcname "+actionType+" -vhost "+vhostName+"]"
           if (isModifyDefaultAction ==1):
                 slaWorkClass = findWorkClass(myWorkClassName,"sla")
                 if (slaWorkClass != ""):
                     AdminConfig.modify(slaWorkClass,[["matchAction", vhostName+":"+actionType]])
           else:
               AdminTask.createServicePolicyWorkClass(taskCmdArgs)
       slaWorkClass=findWorkClass(myWorkClassName,"sla")
       if (slaWorkClass != ""):
          for k in range(0,len(myRule),1):
            if (myRule[k] != None):
               rParts = myRule[k].split("?")
               priority = rParts[0]
               expr = rParts[1]
               actionType = rParts[2]
               validateExpression(expr)
               try:
                  actionValue = rParts[3]
               except IndexError:
                  print "The action argument must be the name of the generic server cluster."
                  errorExit()
               gscName=findGenericServerCluster(actionValue)  
               tcName=findTransactionClass(actionType)              
               print "priority = "+priority+ " expr="+expr+ " actionType="+actionType + " action="+actionValue
               if (slaWorkClass != "" and tcName != "" and gscName != ""):
                   wcRuleAttributes = [["matchAction",actionType+":"+actionValue],
                                       ["matchExpression",expr],
                                       ["priority",priority]]
                   slaRules=AdminConfig.create("MatchRule",slaWorkClass,wcRuleAttributes)
               #print "the len size of rules is %i" %(len(myRule))
          if (myURIS == ""):
             uriExpression = ""
          else:
             uriExpression = ""
             for uri in myURIS.split("?"):
                 if (uriExpression == ""):
                    uriExpression = uri
                 else:
                    uriExpression = uriExpression + "," + uri  
          wcModuleAttributes = [ ["moduleName",myOdrName],
                                 ["matchExpression",uriExpression],
                                 ["id",myWorkClassName+":!:"+myOdrName+":!:"+myOdrNode]]
          slaWorkClassModule=AdminConfig.create("WorkClassModule",slaWorkClass,wcModuleAttributes)
   else:
      if (vhostName == ""):
         errMsg = " virtual host."
      elif (odrNode == ""):
         errMsg = " ODR node."
      elif (odrName == ""):
         errMsg = " ODR."
      elif (tcName == ""):
         errMsg = " transaction class name."
      elif (vhostName == "" and odrNode == ""):
         errMsg = " the Virtual host and ODR node."
      elif (odrNode == "" and odrName == "" and tcName == ""):
         errMsg = " ODR node, ODR, and transaction class name."
      else:
         errMsg = "check command arguments and re-issue command."
      print "The specified arguments are not found in this WAS deployment: "+errMsg
      errorExit()

##
# Creates a WorkClass with user rules 
# for Routing policy
##
def createAppRoutingWorkClass():
   routingWorkClass=""
   routingWorkClassModule=""
   routingRules=""
   actionValue=""
   oldURI=""
   appLongName=findDeployment(myAppName)
   appShortName=myAppName
   #cell=AdminConfig.list("Cell") 
   if (appLongName!=""):
       if (myDefaultAction != ""):
           dParts = myDefaultAction.split("?")
           actionType = dParts[0]
           actionValue = ""
           try:
              actionValue = dParts[1]
           except IndexError:
               if (actionValue == "" and actionType.startswith("permit")):
                   actionValue = myAppName
               else:
                   errorExit() 
           taskCmdArgs =  "[-appname "+ appShortName+" -wcname "+myWorkClassName+" -type "+myProtocolType+" -actiontype "+actionType+" -action "+actionValue+"]"
           if (isModifyDefaultAction ==1):
                 routingWorkClass = findAppRoutingWorkClass(myWorkClassName)
                 if (routingWorkClass != ""):
                    AdminConfig.modify(routingWorkClass,[["matchAction", actionType+":"+actionValue]])
           else:
                 AdminTask.createRoutingPolicyWorkClass(taskCmdArgs)
                 
       routingWorkClass = findAppRoutingWorkClass(myWorkClassName)
       if (routingWorkClass != ""):
          for k in range(0,len(myRule),1):
             if (myRule[k] != None):
               rParts = myRule[k].split("?")
               priority = rParts[0]
               expr = rParts[1]
               actionType = rParts[2]
               actionValue = ""
               validateExpression(expr)
               try:
                  actionValue = rParts[3]
               except IndexError:
                  if (actionValue == "" and actionType.startswith("permit")):
                     actionValue = myAppName
                  else:
                      print "You have specified a routing actiontype of either reject or redirect without the appropriate action."
                      errorExit()
               print "priority = "+priority+ " expr="+expr+ " actionType="+actionType + " action="+actionValue
               if (routingWorkClass != ""):
                   wcRuleAttributes = [["matchAction",actionType+":"+actionValue],
                                       ["matchExpression",expr],
                                       ["priority",priority]]
                   routingRules=AdminConfig.create("MatchRule",routingWorkClass,wcRuleAttributes)
                   #print "the len size of rules is %i" %(len(myRule))
          if (myWebModule != ""):
             moduleNameValue=myWebModule+".war"
             modules = AdminApp.listModules(myAppName)
             mList = modules.split("\n")
             found = 0
             for mod in mList:
                mName = mod.split("#")[1].split("+")[0]
                if (mName == moduleNameValue):
                   found = 1
                   break
                else:
                   moduleNameValue=myWebModule+".jar"
                   if (mName == moduleNameValue):
                       found = 1
                       break
             if (found == 1):
                   if (myURIS == ""):
                       uriExpression = ""
                   else:
                       uriExpression = ""
                       for uri in myURIS.split("?"):
                           if (uriExpression == ""):
                              uriExpression = uri
                           else:
                              uriExpression = uriExpression + "," + uri
                   wcmId = ""
                   fExpression=""
                   if (uriExpression != "*"):
                       fExpression=uriExpression
                   wcmList = AdminConfig.showAttribute(routingWorkClass, "workClassModules").replace("[","").replace("]","").split(" ")
                   for wcm in wcmList:
                      if (wcm == ""):
                        break
                      if (AdminConfig.showAttribute(wcm, "moduleName") == moduleNameValue):
                         wcmId = wcm
                         break   
                   if (wcmId == ""):
                          wcModuleAttributes = [ ["moduleName",moduleNameValue],
                                                 ["matchExpression",uriExpression],
                                                 ["id",myWorkClassName+":!:"+appShortName+":!:"+moduleNameValue]]
                          routingWorkClassModule=AdminConfig.create("WorkClassModule",routingWorkClass,wcModuleAttributes)
                   else:
                          oldURI = AdminConfig.showAttribute(wcm,"matchExpression").replace("[","").replace("]","") 
                          fExpression =  oldURI + "," + fExpression
                          AdminConfig.modify(wcmId,[["matchExpression", fExpression]])
             else:
                print "The workclass webModule element was not created or modified for '"+myWebModule + "' in '"+myAppName + "'"
       else:
          print "The workclass '"+myWorkClassName +"' specified does not exist in this WAS deployment, so no other operation was performed."  
   else:
      print "The application '"+myAppName +"' specified does not exist in this WAS deployment"
      errorExit()
##
# Displays URI for an HTTP WorkClass
#
def listHTTPWorkClassURI():
  checkForRequiredOptions()
  wc=findAppRoutingWorkClass(myWorkClassName)
  if (wc != ""):
    wcmList = AdminConfig.showAttribute(wc, "workClassModules").replace("[","").replace("]","").split(" ")
    for wcm in wcmList:
      if (wcm == ""):
         break
      wcmId = AdminConfig.showAttribute(wcm,"id")
      wcmNames = wcmId.split(":!:")
      if (wcmNames[1]==myAppName and wcmNames[2].startswith(myWebModule)):
           wcmURIS = AdminConfig.showAttribute(wcm,"matchExpression").split(",")
           print "URIs for routing policy work class."
           for wcmURI in wcmURIS:
              if (not wcmURI[0] == '/'):
                  separator = "/"
              else:
                  separator = ""
              print "/" +myAppName +separator+wcmURI
  # now look for service policy under the same workclass name
  #
  wc=findAppSLAWorkClass(myWorkClassName)
  if (wc != ""):
    wcmList = AdminConfig.showAttribute(wc,"workClassModules").replace("[","").replace("]","").split(" ")
    for wcm in wcmList:
      if (wcm == ""):
         break
      wcmId = AdminConfig.showAttribute(wcm,"id")
      wcmNames = wcmId.split(":!:")
      if (wcmNames[1]==myAppName and wcmNames[2].startswith(myWebModule)):
           wcmURIS = AdminConfig.showAttribute(wcm,"matchExpression").split(",")
           print "URIs for service policy work class."
           for wcmURI in wcmURIS:
              if (not wcmURI[0] == '/'):
                  separator = "/"
              else:
                  separator = ""
              print "/" +myAppName +separator +wcmURI
#
# A function for unmapping URI associated with a web module under a given enterprise
# application
#
def removeWorkClassURI():
    checkForRequiredOptions()
    # make sure URI input is well-formed
    uriList = myDeleteURIS.split("?")
    for uri in uriList:
        if (not uri[0] == '/'):
            uri = "/"+uri
    if (myWorkClassType == "" or myWorkClassType == "ARP"):
       wc=findAppRoutingWorkClass(myWorkClassName)
    else:
       wc=findAppSLAWorkClass(myWorkClassName) 
    if (wc != ""):
       newURIs=""
       wcmList = AdminConfig.showAttribute(wc, "workClassModules").replace("[","").replace("]","").split(" ")
       for wcm in wcmList:
           if (wcm == ""):
               break
           wcmId = AdminConfig.showAttribute(wcm,"id")
           wcmNames = wcmId.split(":!:")
           if (wcmNames[1]==myAppName and wcmNames[2].startswith(myWebModule)):
               wcmURIS = AdminConfig.showAttribute(wcm,"matchExpression").split(",")
               for wcmURI in wcmURIS: 
                   for inURI in uriList:
                      if (wcmURI != inURI):
                         if (newURIs == ""):
                            newURIs = wcmURI
                         else:
                            newURIs = newURIs + "," + wcmURI 
               print "The new web module expression is: " + newURIs
               AdminConfig.modify(wcm,[["matchExpression", newURIs]])
    else:
        print ""
        print "The work class, '" + myWorkClassName + "', specified was not found."
        errorExit()  
##
# Creates a WorkClass with user rules 
# for SLA policy
##
def createAppSLAWorkClass():
   slaWorkClass=""
   slaWorkClassModule=""
   slaRules=""
   appLongName=findDeployment(myAppName)
   appShortName=myAppName
   oldURI=""
   protocolType = setProtocolForWorkClass()
   if (appLongName!=""):
       if (myDefaultAction != ""):
           dParts = myDefaultAction.split("?")
           txclsName = dParts[0]
           tcName=findTransactionClass(txclsName)
           if (tcName != ""):   
                 wclassAttributes = [["matchAction",tcName],
                                     ["name",myWorkClassName],
                                     ["type",protocolType],
                                     ["description","This is a custom workclass."]]
                 if (isModifyDefaultAction ==1):
                     slaWorkClass = findAppSLAWorkClass(myWorkClassName)
                     if (slaWorkClass != ""):
                        AdminConfig.modify(slaWorkClass,[["matchAction", tcName]])
                 else:
                     slaWorkClass=AdminConfig.create("WorkClass",appLongName,wclassAttributes)
       if (slaWorkClass == ""):
          slaWorkClass=findAppSLAWorkClass(myWorkClassName)
       if (slaWorkClass != ""):
           for k in range(0,len(myRule),1):
               if (myRule[k] != None):
                   rParts = myRule[k].split("?")
                   priority = rParts[0]
                   expr = rParts[1]
                   txclsName = rParts[2]
                   validateExpression(expr)
                   print "priority = "+priority+ " expr="+expr+ " TransactionClass="+txclsName
                   tcName=findTransactionClass(txclsName)
                   if (tcName != ""):
                       wcRuleAttributes = [["matchAction",tcName],
                                           ["matchExpression",expr],
                                           ["priority",priority]]
                       slaRules=AdminConfig.create("MatchRule",slaWorkClass,wcRuleAttributes)
           if (myWebModule != ""):
               moduleNameValue=myWebModule+".war"
               modules = AdminApp.listModules(myAppName)
               mList = modules.split("\n")
               found = 0
               for mod in mList:
                  mName = mod.split("#")[1].split("+")[0]
                  if (mName == moduleNameValue):
                     found = 1
                     break
                  else:
                     moduleNameValue=myWebModule+".jar"
                     if (mName == moduleNameValue):
                        found = 1
                        break
               if (found == 1):
                   if (myURIS == ""):
                       uriExpression=""
                   else:
                       uriExpression = ""
                       for uri in myURIS.split("?"):
                           if (myEJBName == ""):
                               if (uriExpression == ""):
                                   uriExpression = uri
                               else:
                                   uriExpression = uriExpression + "," + uri
                           else:                               
                               if (uriExpression == ""):
                                   uriExpression = myEJBName+":"+uri
                               else:
                                   uriExpression = uriExpression + "," + myEJBName+":"+uri

                   wcmId = ""
                   fExpression=""
                   if (uriExpression != "*"):
                       fExpression=uriExpression
                   wcmList = AdminConfig.showAttribute(slaWorkClass, "workClassModules").replace("[","").replace("]","").split(" ")
                   for wcm in wcmList:
                      if (wcm == ""):
                        break
                      if (AdminConfig.showAttribute(wcm, "moduleName") == moduleNameValue):
                         wcmId = wcm
                         break   
                   if (wcmId == ""):
                          wcModuleAttributes = [ ["moduleName",moduleNameValue],
                                                 ["matchExpression",uriExpression],
                                                 ["id",myWorkClassName+":!:"+appShortName+":!:"+moduleNameValue]]
                          routingWorkClassModule=AdminConfig.create("WorkClassModule",slaWorkClass,wcModuleAttributes)
                   else:
                          oldURI = AdminConfig.showAttribute(wcm,"matchExpression").replace("[","").replace("]","") 
                          fExpression = oldURI + "," +fExpression
                          AdminConfig.modify(wcmId,[["matchExpression", fExpression]])
               else:
                  print "The workclass webModule element was not created or modified for '"+myWebModule + "' in '"+myAppName + "'"
       else:
          print "The workclass '"+myWorkClassName +"' specified does not exist in this WAS deployment, use the -setdefaultaction option to create a new workclass."
   else:
      print "The application '"+myAppName+ "' specified does not exist in this WAS deployment."
      errorExit()

def createSystemAppRoutingWorkClass():
   routingWorkClass=""
   actionValue=""
   oldURI=""
   appShortName=myAppName 
   print "the short appname = " + appShortName
   if (appShortName!=""):
       if (myDefaultAction != ""):
           dParts = myDefaultAction.split("?")
           actionType = dParts[0]
           actionValue = ""
           try:
              actionValue = dParts[1]
           except IndexError:
               if (actionValue == "" and actionType.startswith("permit")):
                   actionValue = appShortName
               else:
                   errorExit() 
           taskCmdArgs =  "[-appname "+ appShortName+" -wcname "+myWorkClassName+" -type "+myProtocolType+" -actiontype "+actionType+" -action "+actionValue+"]"                
           if (isModifyDefaultAction == 1):
                 routingWorkClass = findSystemAppRoutingWorkClass(myWorkClassName)
                 AdminConfig.modify(routingWorkClass,[["matchAction", actionType+":"+actionValue]])
           else:
                 routingWorkClass = AdminTask.createSystemRoutingPolicyWorkClass(taskCmdArgs)          
       
       else:
          print "The workclass '"+myWorkClassName +"' specified does not exist or could not be created in this WAS deployment."
   else:
      errorExit()	 
          
def createSystemAppSLAWorkClass():
   slaWorkClass=""
   appShortName=myAppName
   protocolType = setProtocolForWorkClass()
   if (appShortName!=""):
       if (myDefaultAction != ""):
           dParts = myDefaultAction.split("?")
           txclsName = dParts[0]
           tcName=findTransactionClass(txclsName)
           print "Tx Class = " + tcName
           if (tcName != ""):   
                 wclassAttributes = [["matchAction",tcName],
                                     ["name",myWorkClassName],
                                     ["type",protocolType],
                                     ["description","This is a custom workclass."]]  
                 taskCmdArgs =  "[-appname "+ appShortName+" -wcname "+myWorkClassName+" -type "+myProtocolType+ " -tcname "+tcName +" -appEditionName  "+appShortName+"]"
                 if (isModifyDefaultAction == 1):
                        slaWorkClass = findSystemAppRoutingWorkClass(myWorkClassName)
                        AdminConfig.modify(slaWorkClass,[["matchAction", tcName]])
                 else:
                       slaWorkClass=AdminTask.createSystemServicePolicyWorkClass(taskCmdArgs)    
       else:
          print "The workclass '"+myWorkClassName +"' specified does not exist or could not be created in this WAS deployment."
   else:
      errorExit()	   
      
def removeAppRoutingWorkClass():
   appLongName=""
   if (isSystemApp == 0):
      appLongName=findDeployment(myAppName)
   if (appLongName!="" or isSystemApp == 1):
      wcToRemove=findAppRoutingWorkClass(myWorkClassName)
      if (wcToRemove != ""):
          print "Deleting enterprise application for routing workclass: " + wcToRemove
          AdminConfig.remove(wcToRemove)
      else:
         print "The workclass '"+myWorkClassName +"' specified does not exist in this WAS deployment."
         errorExit()
   else:
      print "The application '"+myAppName +"' specified does not exist in this WAS deployment."
      errorExit()
       
def removeAppSLAWorkClass():
   appLongName=""
   if (isSystemApp == 0):
      appLongName=findDeployment(myAppName)
   if (appLongName!="" or isSystemApp == 1):
      wcToRemove=findAppSLAWorkClass(myWorkClassName)
      if (wcToRemove != ""):
         print "Deleting enterprise application for SLA workclass: '" + wcToRemove +"'"
         AdminConfig.remove(wcToRemove)
      else:
         print "The workclass '"+myWorkClassName +"' specified does not exist in this WAS deployment."
         errorExit()
   else:
      print "The application '"+myAppName +"' specified does not exist in this WAS deployment."
      errorExit()
            
def removeGSCRoutingWorkClass():
    odrName=findProxyServer(myOdrName)
    wcToRemove=findWorkClass(myWorkClassName,"routing")
    if (odrName != "" and wcToRemove != ""):
        print "Deleting generic server cluster routing workclass: '" + wcToRemove +"'"
        AdminConfig.remove(wcToRemove)
    else:
       print "The ODR '"+odrName + "' or workclass '"+myWorkClassName+ "' specified does not exist in this WAS deployment."
       errorExit()
      
def removeGSCSLAWorkClass():
    odrName=findProxyServer(myOdrName)
    wcToRemove=findWorkClass(myWorkClassName,"sla")
    if (odrName != "" and wcToRemove != ""):
        print "Deleting generic server cluster SLA workclass: '" + wcToRemove + "'"
        AdminConfig.remove(wcToRemove)
    else:
       print "The ODR '"+odrName + "' or workclass '"+myWorkClassName+ "' specified does not exist in this WAS deployment."
       errorExit()
       
def processAppWorkClassRequest():
    checkForRequiredOptions()
    # if workclass exists, then webmodule, uris, can be absent
    if (myRule[0] == None):
        print "Creating work class with only default rule and match expression or modifying existing work class."
    if (myRule[0] != None and (myRule[0].find("permit") > -1 or myRule[0].find("redirect") > -1 or myRule[0].find("permitsticky") > -1 or myRule[0].find("reject") > -1)):
        createAppRoutingWorkClass()
        print "*****************************************************************************************"
        print ""
        print "Work Class for routing created or updated."
    elif (myDefaultAction != "" and (myDefaultAction.find("permit") > -1 or myDefaultAction.find("redirect") > -1 or myDefaultAction.find("permitsticky")> -1 or myDefaultAction.find("reject") > -1)):
        createAppRoutingWorkClass()
        print "*****************************************************************************************"
        print ""
        print "Set Default Action for routing policy."
    elif (myURIS != "" and myWorkClassType == "ARP"):
        print "Updating Routing WorkClass URIs."
        createAppRoutingWorkClass()   
    else:
        createAppSLAWorkClass()
        print "*****************************************************************************************"
        print ""
        print "Work Class for service policy created or updated."
    print "*****************************************************************************************"    	
    
    
def processGSCWorkClassRequest():
    checkForRequiredOptions()
    if (myRule[0] == None):
        print "Creating work class for generic server cluster with only default rule and action or modifying existing work class."
    if (myRule[0] != None and (myRule[0].find("permit") > -1 or myRule[0].find("redirect") > -1 or myRule[0].find("permitsticky") > -1 or myRule[0].find("reject") > -1)):
        createGSCRoutingWorkClass()
        print "*****************************************************************************************"
        print ""
        print "Work Class for routing created or updated."
    elif (myDefaultAction != "" and (myDefaultAction.find("permit") > -1 or myDefaultAction.find("redirect") > -1 or myDefaultAction.find("permitsticky")> -1 or myDefaultAction.find("reject") > -1)):
        createGSCRoutingWorkClass()
        print "*****************************************************************************************"
        print ""
        print "Work Class for GSC routing created with default action and expression."
    else:
       createGSCSLAWorkClass()
       print "*****************************************************************************************"
       print ""
       print "Work Class for SLA created or updated." 
    print "*****************************************************************************************" 

def processSystemAppRequest():
    checkForRequiredOptions()
    # if workclass exists, then webmodule, uris, can be absent
    if (myDefaultAction != "" and (myDefaultAction.find("permit") > -1 or myDefaultAction.find("redirect") > -1 or myDefaultAction.find("permitsticky")> -1 or myDefaultAction.find("reject") > -1)):
        createSystemAppRoutingWorkClass()
        print "*****************************************************************************************"
        print ""
        print "Set Default Action for routing policy."  
    else:
        createSystemAppSLAWorkClass()
        print "*****************************************************************************************"
        print ""
        print "Work Class for service policy created or updated for system application."
    print "*****************************************************************************************"    	


def validateExpression(anExpr):
    try:
        Expression(anExpr)
    except RuleParserException, msg:
        print "ERROR: The rule expression is invalid: ", msg
        errorExit()
        
def normalExit():
    java.lang.System.exit(0)
 
def errorExit():
    java.lang.System.exit(1)
 
def showRemoveWCHelp():
    print """
        DESCRIPTION
        This command option is used to remove an existing application work class. 
        
	    Usage: wsadmin 
	                  [-lang jython]
	                  [-f workclassoperations.py]
	                  [-wctype ARP|ASP|GRP|GSP]
	                  [-odrname odrName] 
	                  [-wcname workClassName]
	                  [-appname applName]
	                  [-deletewc]
	                  [-systemapp]
	                  
	    COMMAND LINE OPTIONS
	      -appname appName is used to specify the name of the enterprise application without the .ear extension. This option is required if deleting enterprise application work class.
	      -deletewc is the operation to be performed, deletes the specified work class.
	      -odrname odrName is used to specify the name of your On Demand Router (ODR) proxy server. This option is required if deleting generic server cluster work class. 
	      -systemapp is used to indicate whether the application specified is a system application.
	      -wcname workClassName is used to specify the name of the workclass, no spaces are allowed in the work class name. This option is required.
	      -wctype takes a constant value of either ARP for enterprise application routing policy work class, ASP for enterprise application service policy work class, GRP for generic server cluster routing policy work class, or GSP for generic server cluster service policy work class. There is no default, so a value must be specified.
	    
	    EXAMPLES
	    Example 1, wsadmin -lang jython -f workclassoperations.py -deletewc -wctype GSP -odrname ODR -wcname CustomWorkClass
	    The example above is used to remove a work class named CustomWorkClass for a service policy in the generic server cluster identified through the ODR name.
	    
	    Example 2, wsadmin -lang jython -f workclassoperations.py -deletewc -wctype ARP -appname A -wcname CustomWorkClass
	    The example above is used to remove a work class named CustomWorkClass for a routing policy in the enterprise application called A.

    """
    java.lang.System.exit(1)      
def showHelp():    
     print """
        DESCRIPTION
            This command is used to create or update work classes for a deployed application and/or generic server cluster, list rules in a work class, get default action defined in a work class, delete rule in a work class, add URI to a work class web module, modify URIs, list URIs, or remove work classes. 
            Each invocation can only create one work class; either for routing policy or for service policy. To create a set of work classes for enterprise application and generic server cluster, the command must be executed at least twice. While some options like -wctype and -wcname are required for all command invocations, others are required based on the combination of options and the intended operation. The examples below will aid you in knowing the required options for each operation.
            In general, when the command usage is displayed because of incorrect or incomplete input options, the cause of failure will be printed on top of the usage message, so scroll up to find out specific error messages. Also, in the examples, certain characters have been escaped to prevent the command-line processor from interpreting them as special charaters. In such cases, there may be variations based on the operating system. For example, on Windows, to escape the percent sign (%), the character sequence (\%%) is used while on Unix type operating systems, to escape the percent sign, the escape sequence is (\%).
            
	    Usage: wsadmin 
	                   [-lang jython]
	                   [-f workclassoperations.py]
	                   [-odrname odrName]
	                   [-odrnode odrNode]
	                   [-wcname workClassName]
	                   [-appname appName]
	                   [-webmodule webModule]
	                   [-addmember ["member1?member2?..."]]
	                   [-type HTTP|SOAP]
	                   [-vhost virtualHost]
	                   [-wctype APP|GSC|ARP|ASP|GRP|GSP]
	                   [-addrule ["priority?expr?actiontype?action"]]
	                   [-deleterule [priority]]
	                   [-deletemember ["member1?member2?..."]]
	                   [-deletewc]
	                   [-listrule]
	                   [-listmembers]
	                   [-setdefaultaction ["actiontype?action"]]
	                   [-modifydefaultaction ["actiontype?action"]]
	                   [-getdefaultaction]
	                   [-systemapp]
	                   [-both]
	    
	    EXAMPLES
	       Example 1: wsadmin -lang jython -f workclassoperations.py -appname A -wcname CustomWorkClass -type HTTP -wctype APP -webmodule myWebModule -addmember "/test1?/test2/*" -addrule "1?clienthost=\'localhost\' and serverhost like \'\%%.ibm.com\'?permit" -setdefaultaction "reject?404"
	       The example above is used to create enterprise application work class for routing policy. It has the default action of reject with a reject code of 404 and priority of 2. If issuing the command above on Unix base operating system, the escape sequence (\%%) should be replaced by (\%).
	       
	       Example 2: wsadmin -lang jython -f workclassoperations.py -appname A -wcname Custom2WorkClass -type HTTP -wctype APP -webmodule myWebModule -setdefaultaction "Default_TC" -addmember "/test1?/test2/*" -addrule "1?clienthost=\'localhost\' and serverhost like \'\%%.ibm.com\'?Default_TC"
	       The example above is used to create enterprise application work class for service policy. If issuing the command above on Unix base operating system, the escape sequence (\%%) should be replaced by (\%).
	       
	       Example 3: wsadmin -lang jython -f workclassoperations.py -odrname ODR -odrnode myNode01 -wcname GSCWorkClass  -vhost default_host -wctype GSC -setdefaultaction "permit?myGSC"
	       The example above is used to create work class for routing policy with a default type of HTTP. Its default action rule has a priority of 0, and action type of permit, and an action of the generic server cluster name for that requests are routed to.
	    
	       Example 4: wsadmin -lang jython -f workclassoperations.py -odrname ODR -odrnode myNode01 -wcname GSCWorkClass -vhost default_host -wctype GSC -addrule "1?serverhost=\'localhost\' and serverhost like \'\%%.ibm.com\'?TransactionClassA" -setdefaultaction "TransactionClassB"
	       The example above is used to create work class for the service policy with a default type of HTTP. If issuing the command above on Unix base operating system, the escape sequence (\%%) should be replaced by (\%).
	    
	       Example 5: wsadmin -lang jython -f workclassoperations.py -deleterule 1 -wcname CustomWorkClass -both -appname A -wctype APP"
	       The example above is used to delete a rule with priority 1 in work class named CustomWorkClass for both (-both option) routing and service policies.
	       
	       Example 6: wsadmin -lang jython -f workclassoperations.py -listrule -wcname CustomWorkClass -both -appname A -wctype APP"
	       The example above is used to list rules in work class named CustomWorkClass for both (-both option) routing and service policies for the enterprise application A.
	       
	       Example 7: wsadmin -lang jython -f workclassoperations.py -listrule -wcname CustomWorkClass -both -odrname ODR -odrnode theTestNode01 -wctype GSC"
	       The example above is used to list rules in work class named CustomWorkClass for both (-both option) routing and service policies for the generic server cluster.
	       
	       Example 8: wsadmin -lang jython -f workclassoperations.py -getdefaultaction -wcname CustomWorkClass -both -appname A -wctype APP"
	       The example above is used to get default actions in work class named CustomWorkClass for both (-both option) routing and service policies.
	       
	       Example 9: wsadmin -lang jython -f workclassoperations.py -deletewc -wcname CustomWorkClass -appname A -wctype ARP"
	       The example above is used to delete a routing policy work class named CustomWorkClass under the enterprise application A.
	       
	       Example 10: wsadmin -lang jython -f workclassoperations.py -deletewc -wcname CustomWorkClass -appname A -wctype ASP"
	       The example above is used to delete a service policy work class named CustomWorkClass under the enterprise application A edition.
	       
	       Example 11: wsadmin -lang jython -f workclassoperations.py -deletewc -wcname CustomWorkClass -odrname myODR -wctype GSP"
	       The example above is used to delete a service policy work class named CustomWorkClass under the ODR named myODR.
	       
	       Example 12: wsadmin -lang jython -f workclassoperations.py -deletewc -wcname CustomWorkClass -odrname myODR -wctype GRP"
	       The example above is used to delete a routing policy work class named CustomWorkClass under the ODR named myODR.
	       
	       Example 13: wsadmin -lang jython -f workclassoperations.py -listmembers -wcname CustomWorkClass -appname A -webmodule microwebapp"
	       The example above is used to list URIs in work class named CustomWorkClass for application A and web module microwebapp.
	       
	       Example 14: wsadmin -lang jython -f workclassoperations.py -deletemember "/myURI/help.jsp" -wcname CustomWorkClass -appname A -webmodule microwebapp"
	       The example above is used to delete URIs in work class named CustomWorkClass for application A and web module microwebapp. Use the -wctype ARP or ASP to distinquish between routing and service policies work class. If -wctype is not used, the routing policy work class is used.
	       
	       Example 15: wsadmin -lang jython -f workclassoperations.py -appname A -webmodule microwebapp -wcname Custom2WorkClass -wctype APP -type HTTP -addmember "/test1?/test2/*" -setdefaultaction "redirect?http://www.ibm.com" -addrule "1?clienthost=\'localhost\' and serverhost like \'\%%.ibm.com\'?permit?A"  -addrule "2?protocol IN (\'HTTP\',\'HTTPS\')?permitsticky?A"
	       The example above is used to create enterprise application work class for routing policy with two additional rules and the default. If issuing the command above on Unix base operating system, the escape sequence (\%%) should be replaced by (\%).
	       
	       Example 16: wsadmin -lang jython -f workclassoperations.py -addmember "/myURI/help.jsp" -wcname CustomWorkClass -appname A -webmodule microwebapp -wctype ARP"
	       The example above is used to add URIs to an existing work class named CustomWorkClass for application A and web module microwebapp. Use the -wctype ARP or ASP to distinquish between routing and service policies work class.
	       
	       Example 17: wsadmin -lang jython -f workclassoperations.py -appname A -wcname CustomWorkClass -type SOAP -wctype APP -webmodule myWebModule -addmember "RemoteTest:addJob?RemoteTest:removeJob" -addrule "1?clienthost=\'localhost\' and serverhost like \'\%%.ibm.com\'?permit" -setdefaultaction "reject?404"
	       The example above is used to create enterprise application SOAP work class for routing policy. It has the default action of reject with a reject code of 404 and priority of 2. If issuing the command above on Unix base operating system, the escape sequence (\%%) should be replaced by (\%).
	       
	    COMMAND LINE OPTIONS
	    -addrule is used to add or define rules for the created work class. It cannot be used with the -deleterule, -listrule, or -getdefaultaction options on the same command invocation. If adding more than one rule at a time, list them in the order of their priority starting from a low number to the higher number in sequence.
	             'priority' is the priority assigned to the rule being created, it takes on integer value, for example, 0, 1, 2, 3, etc., with 0 being the highest priority.
	             the question mark (?) is a separator and it is required in this sequence.
	             'expr' is the rule expression as documented in the WebSphere Extended Deployment documentation.
	             'actiontype' is the type of action to be taken and has an enumerated values of permit, permitsticky, reject, or redirect. If creating an service policy work class, the actiontype is the transaction class name.
	             'action' its value depends on the actiontype. For example, if actiontype is 'redirect' then the action will be the redirect URL. For example, redirect?http://www.ibm.com. For actiontype of reject,
	             the action will be a reject code. For example, reject?404. For generic server cluster work classes, the action is the name of the generic server cluster.  
	    -appname appName is used to specify the name of the enterprise application without the .ear extension.
	    -addmember is used to provide a collection of member patterns to associate with the work class web module for the enterprise application. Member patterns are separated by the (?) sign and each member pattern must start with a forward slash (/).
	            For SOAP work classes, the format must be "<webService>:<operationName>?<webSerivce>:<operationName>".  This example is for two entries.
   	            For IIOP work classes, the format must be "<ejbName>:<ejbMethod>?<ejbName>:<ejbMethod>".  This example is for two entries.
   	            For JMS work classes, the format must be "<bus>:<destination>?<bust>:<destination>".  This example is for two entries.   	            
	    -both is used to indicate that the -listrule, -deleterule, or the -getdefaultaction operation should be applied for both routing and service policies on the given work class name.
	    -deleterule is used to delete a specific rule from the work class specified using the -wcname option. It cannot be used with the -addrule, -listrule, or -getdefaultaction options on the same command invocation.
	    -deletemember is used to provide a collection of URI patterns to be deleted from the web module's work class. This option is applicable to only enterprise application work classes.
	    -listmembers is used to display URI from the web module element of the work class. This option is applicable to only enterprise application work classes.
	    -deletewc is used to delete a work class. The -wctype with a value of either ARP, ASP, GRP, GSP must also be specified along with this option.
	    -getdefaultaction gets the default actiontype and action defined in the work class specified using the -wcname option. It cannot be used with -addrule, -delete, or -listrule options on the same command invocation.
	    -listrule displays the rules defined in the work class. It cannot be used with -addrule, -deleterule, or -getdefaultaction options on the same command invocation.
	    -modifydefaultaction uses the actiontype and action specified to modify the default action for the existing work class specified using the -wcname option.
	    -odrname odrName is used to specify the name of your On Demand Router (ODR) proxy server.
	    -odrnode odrNode is used to specify the name of your ODR node.
	    -setdefaultaction uses the actiontype and action specified to create a rule with the expression denoted by the asterisk (*) sign and sets the action type and action to the values provided with the option.
	    -systemapp is used to indicate that the application is a system application
	    -type HTTP   if the -type option is not specified, then the default of HTTP will be assumed.
	    -vhost virtualHost  is used to specify the name of the virtual host for the generic server cluster.
	    -webmodule webModule  is the web module name without the .war extension.
	    -wcname workClassName is used to specify the name of the workclass, no spaces are allowed in the work class name. This option is required.
	    -wctype takes a constant value of either APP for enterprise application or GSC for generic server cluster. If the -deletewc option is specified, then acceptable values are ARP for delete application routing work class, ASP for delete application service policy work class, GRP for delete generic server cluster routing policy work class, and GSP for delete generic server service policy work class. There is no default, so a value must be specified.

     """
     errorExit()
#
#
#  main() - 
#
if (len(sys.argv) < 5):
   showHelp()
global myOdrName,myWorkClassName,myOdrNode,myVirtualHost,myActionType,myTCName,myAppName,myRule,myWebModule,myURIS,myDeleteURIS,isPerformBoth,isAppWorkClasses,isListRule,isGetDefaultAction,isModifyDefaultAction,isSystemApp,myEJBName,patternOverlaps
myWhatToCreate=0
isPerformBoth=0
isListRule=0
isListURI=0
isGetDefaultAction=0
isModifyDefaultAction=0
isDeleteWorkClass=0
isDeleteURI=0
isSystemApp=0
myProtocolType=""
myOdrName=""
myWorkClassName=""
myOdrNode=""
myVirtualHost=""
myActionType=""
myTCName=""
myAppName=""
myWebModule=""
myAction=""
myExpression=""
myWorkClassType=""
myDefaultAction=""
myDeleteURIS=""
myURIS=""
myEJBName=""
patternOverlaps=""
myDeleteRulePriority=-1
count=0
myRule=jarray.zeros(9,java.lang.String)
# myWhatToCreate=1: Routing; myWhatToCreate=2:Service Policy; Both:3
for i in range(0,len(sys.argv),1): 
  option = sys.argv[i]
  if (option == "-odrname"):
      myOdrName = sys.argv[i+1].rstrip()
  elif (option == "-odrnode"):
      myOdrNode = sys.argv[i+1].rstrip()
  elif (option == "-wcname"):
      myWorkClassName = sys.argv[i+1].rstrip()
  elif (option == "-ejbName"):
      myEJBName = sys.argv[i+1].rstrip()      
  elif (option == "-deletewc"):
      isDeleteWorkClass = 1
  elif (option == "-systemapp"):
      isSystemApp = 1
  elif (option == "-type"):
      myProtocolType = sys.argv[i+1].rstrip()
  elif (option == "-vhost"):
      myVirtualHost = sys.argv[i+1].rstrip()
  elif (option == "-both"):
      isPerformBoth = 1
  elif (option == "-appname" ):
      myAppName = sys.argv[i+1].rstrip()
  elif (option == "-webmodule"):
      myWebModule = sys.argv[i+1].rstrip()
  elif (option == "-addmember" or option == "-adduri"):
      myURIS = sys.argv[i+1].rstrip()
  elif (option == "-deletemember" or option == "-deleteuri"):
      myDeleteURIS = sys.argv[i+1].rstrip()
      isDeleteURI = 1
  elif (option == "-wctype"):
      myWorkClassType = sys.argv[i+1].rstrip()
      if (myWorkClassType == "APP" or myWorkClassType == "ARP" or myWorkClassType == "ASP" ):
           isAppWorkClasses=1
      else:
           isAppWorkClasses=0
  elif (option == "-listrule"):
      isListRule = 1
  elif (option == "-listmembers" or option == "-listuri"):
      isListURI = 1
  elif (option == "-getdefaultaction"):
      isGetDefaultAction = 1
  elif (option == "-deleterule"):
      myDeleteRulePriority = sys.argv[i+1].rstrip()
      if (not(myDeleteRulePriority.isdigit())):
          print ""
          print "You have specified -deleterule option without passing in a valid priority, priority must be a digit."
  elif (option == "-setdefaultaction"):
      myDefaultAction = sys.argv[i+1].rstrip()
      if (len(myDefaultAction.split("?")) < 1):
          print ""
          print "You have specified -setdefaultaction option without passing in the required arguments."
  elif (option == "-modifydefaultaction"):
      myDefaultAction = sys.argv[i+1].rstrip()
      isModifyDefaultAction = 1
      if (len(myDefaultAction.split("?")) < 1):
          print ""
          print "You have specified -modifydefaultaction option without passing in the required arguments."
  elif (option == "-addrule"):
      myRule[count] = sys.argv[i+1].rstrip().replace("\\","")
      if (len(myRule[count].split("?")) < 3):
          print ""
          print "You have specified -addrule option without passing in the required arguments."
      count = count + 1
      
if(myProtocolType == ""):
     myProtocolType = "HTTP"     

#
# Find out what the caller is trying to do and check to make
# sure they've got the correct options to do what they want
#
if (not (myAppName == "")):
    if ( isSystemApp == 0 and AppTypeDeploymentTask.isLongRunningApp(myAppName)):
        print ""
        print "INFO: Invalid command."
        print "INFO: Long Running Applications do not have scripting support.  No processing was done, goodbye!"
        errorExit()       

#TODO --
# Haven't tested this out yet.
# Also, make sure zos autoFVT for this script is not effected
#
cellName = AdminConfig.showAttribute(AdminConfig.list("Cell"),"name")    
if (myProtocolType == "IIOP" or myProtocolType == "JMS"):
    if (ApplicationUtil.isAppDeployedOnZOS(myAppName, cellName)):
        print ""
        print "INFO: Invalid command."
        print "INFO: "+myProtocolType+" work classes are not supported on this platform at this time.  No processing was done, goodbye!"
        errorExit()        
        
    if (myWorkClassType != "ASP"):
        print ""
        print "INFO: Invalid command."        
        print "INFO: "+myProtocolType+" work classes of "+myWorkClassType+" type are not supported at this time.  No processing was done, goodbye!"
        errorExit()

if (myProtocolType == "JMS"):    
    if (ApplicationUtil.isEJBModule(myAppName, myWebModule+".jar", cellName)):
        if (myEJBName == ""):
            print ""
            print "INFO: Invalid command."
            print "INFO: When specifying an ejb module for a JMS workclass please specify an EJB with the -ejbName parameter."
            errorExit()  
    else:
        if (not(myEJBName == "")):
            print ""
            print "INFO: Invalid command."
            print "INFO: The -ejbName parameter is reserved for JMS work classes and ejb jar modules."
            errorExit()

if (myProtocolType != "HTTP"):
    if (myURIS != ""):
        #matchExpression is not empty so validate
        matchExpression = ""
        valid = "false"
        for match in myURIS.split("?"):
            matchExpression = match
            name = matchExpression.split(":")
            if len(name) == 2:
                valid = "true"
            else:
                valid = "false"
    else:
        #empty matchExpression is valid
        valid = "true"
        
    if valid == "false":
        print ""
        print "INFO: Invalid command."
        
        exampleURIs = "";
        if (myProtocolType == "SOAP"):
            exampleURIs = "<webService>:<operationName>?<webSerivce>:<operationName>"
        elif (myProtocolType == "IIOP"):
            exampleURIs = "<ejbName>:<ejbMethod>?<ejbName>:<ejbMethod>"
        else:
            exampleURIs = "<bus>:<destination>?<bus>:<destination>"
            
        print "INFO: For "+myProtocolType+" work classes, the format needs to be '"+exampleURIs+"' ."
        errorExit()       
    
if (myURIS != ""):
    for match in myURIS.split("?"):
        if (isMatchOverlapping(match) == "true"):
            print ""
            print "INFO: Invalid command."
            print "INFO: the match pattern '"+match+"' has already been mapped for this Enterprise Application/Module."
            errorExit()          
      
if (isDeleteWorkClass == 1 and myWorkClassType == "ARP"):
    if (myAppName == "" or myWorkClassName == ""):
        showRemoveWCHelp()
    checkForReservedWC()
    removeAppRoutingWorkClass()
elif (isDeleteWorkClass == 1 and myWorkClassType == "ASP"):
    if (myAppName == "" or myWorkClassName == ""):
        showRemoveWCHelp()
    checkForReservedWC()
    removeAppSLAWorkClass()
elif (isDeleteWorkClass == 1 and myWorkClassType == "GRP"):
   if (myOdrName == "" or myWorkClassName == ""):
        showRemoveWCHelp()
   checkForReservedWC()
   removeGSCRoutingWorkClass()	
elif (isDeleteWorkClass == 1 and myWorkClassType == "GSP"):
   if (myOdrName == "" or myWorkClassName == ""):
        showRemoveWCHelp()
   checkForReservedWC()
   removeGSCSLAWorkClass()
elif (isSystemApp == 1 and isModifyDefaultAction):
   processSystemAppRequest() 
elif (isDeleteURI == 1):
   if (myDeleteURIS == "" or myDeleteURIS.startswith("-")):
        print ""
        print "You have specified -deletemember option without passing in the required argument."
   isAppWorkClasses=1
   removeWorkClassURI()
elif (isListURI == 1):
    isAppWorkClasses=1
    listHTTPWorkClassURI()
    normalExit()	
elif (isListRule == 1):
    processRulesForWorkClass()
    normalExit()
elif (isGetDefaultAction == 1):
    processRulesForWorkClass()
    normalExit()
elif (myDeleteRulePriority != -1):
    deleteWorkClassRule()
    normalExit()
elif (isAppWorkClasses ==1 and (myRule[0] != None or myDefaultAction != "")):
    # process app workclasses
    processAppWorkClassRequest()
elif (isAppWorkClasses == 0 and (myRule[0] != None or myDefaultAction != "")):
    # process GSC workclass
    processGSCWorkClassRequest()
elif (myURIS != "" and (myWorkClassType == "ARP" or myWorkClassType == "ASP")):
    processAppWorkClassRequest()
else:
    print ""
    print "INFO: Invalid command."    
    print "INFO: You have not supplied valid options, no processing was done, goodbye!"
    
print "Saving workspace"
AdminConfig.save()
print "finished."
